def family(famille):
    if famille in ('ua', 'ua_qc'):
        FAM = "Radiosondes"
        VCOORD = " round(vcoord/025000.)*02500 "
        VCOORD = " round(vcoord/20000.)*20000 "
    #    VCOORD = " round(vcoord/05000.)*05000 "
        STATB = "OBS_ERROR"
        STATB = " 0."
        VCOCRIT = " and  id_stn like '%' "
        VCOCRIT = "    "
        elem = "11004,11003,12001,12192"
        VCOTYP = 'PRESSION'
    elif famille in ('gp', 'gp_qc', 'gpssfc_b'):
        FAM = "GBGPS"
        VCOTYP = 'SURFACE'
        VCOCRIT = "  "
        VCOORD = " 9999    "
        VCOORD = " round(lat/10)*10    "
        STATB = "OBS_ERROR"
        STATB = "0. "
        elem = "15031 "
    elif famille == 'synop_b':
        FAM = "SHIPS"
        VCOTYP = 'SURFACE'
        VCOORD = "'SURF'   "
        STATB = "OBS_ERROR"
        VCOCRIT = "    "
    elif famille in ('sc', 'sc_qc'):
        FAM = "SCATS"
        VCOTYP = 'SURFACE'
        VCOORD = " 9999    "
        VCOORD = " round(lat/10)*10    "
        VCOCRIT = "      "
        STATB = "OBS_ERROR"
        STATB = "0. "
        elem = "11215,11216"
    elif famille in ('ro', 'ro_qc', 'gpsocc'):
        FAM = "GPSRO"
        VCOTYP = 'HAUTEUR(metres)'
        VCOORD = " round(vcoord/10000.)*10000 "
        VCOORD = " round(vcoord/1000.)*1000 "
        STATB = " 0. "
        VCOCRIT = "      "
        elem = "15036"
    elif famille in ('mwhs2', 'mwhs2_qc'):
        FAM = "MWHS2"
        VCOORD = "  vcoord "
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        VCOCRIT = "      "
        elem = "12163"
    elif famille in ('to_amsua_qc', 'to_amsua', 'to_amsua_allsky', 'to_amsua_allsky_qc'):
        FAM = "AMSUA"
        VCOORD = "  vcoord "
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR" 
        VCOCRIT = "      "
        elem = "12163"
    elif famille in ('to_amsub_qc', 'to_amsub_allsky'):
        FAM = "AMSUB"
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        VCOORD = "  vcoord "
        VCOCRIT = "  "
        elem = "12163"
    elif famille in ('ssmis_qc', 'ssmis'):
        FAM = "SSMIS"
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        VCOORD = "  vcoord "
        elem = "12163"
        VCOCRIT = "  "
    elif famille in ('iasi', 'iasi_qc'):
        FAM = "IASI"
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        VCOORD = "  vcoord "
       # VCOORDLIST = "32,38,44,50,57,63,76,79,82,87,104,109,116,122,128,135,141,154,160,167,173,180,185,199,205,212,213,214,217,219,224,226,230,232,236,239,243,246,249,252,262,265,269,275,282,299,300,323,327,329,335,347,350,354,356,360,366,371,373,375,377,379,381,389,404,407,410,414,416,426,428,432,434,445,457,515,546,552,566,571,573,646,662,668,756,867,921,1027,1046,1121,1133,1191,1194,1271,2019,2094,2119,2213,2321,2398,2907,2944,2951,2977,2990,2993,3008,3027,3030,3049,3058,3087,3107,3110,3127,3151,3160,3228,3263,3303,3432,3467,3497,3499,3518,3610,3646,3673,3710,3763,4920,4991,5371,6135,6149,6158,6161,6174,6205,6209,6213,6317"
        VCOCRIT = " "
        elem = "12163"
    elif famille in ('ch','ch_db'):
        FAM = "OZONE"
        VCOTYP = 'LEVEL'
        VCOORD = "  vcoord    "
        elem = "15198,15008"
        FONCTION = "(100.*omp/obsvalue)"
        FONCTION2 = "(omp*0.)"
        STATB = '    '
        STN_IDS = " id_stn in ('AURA-MLS') and vcoord = 100"
        VCOCRIT = '    '
    elif famille in ('crisfsr1_qc', 'crisfsr2_qc', 'cris'):
        FAM = "CRISFSR"
        VCOORD = "  vcoord "
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        elem = "12163"
        VCOCRIT = "    "
    elif famille in ('atms_allsky', 'atms_qc', 'atms'):
        
        FAM = "ATMS"
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        VCOORD = "  vcoord " 
        VCOCRIT = "    "
        elem = "12163"
    elif famille in ('csr', 'csr_qc'):
        FAM = "CSR"
        VCOTYP = 'CANAL'
        STATB = "BIAS_CORR"
        VCOCRIT = "  "
        VCOORD = "  vcoord "
        elem = "12163"
    elif famille in ('ai', 'ai_qc','012_ai.sqlite' ):
        FAM = "AIRCRAFTS"
        VCOTYP = 'PRESSION'
        VCOORD = " round(vcoord/20000.)*20000 "
       # VCOORD = " round(vcoord/05000.)*05000 "
        STATB = "BIAS_CORR"
        VCOCRIT = " and vcoord > 00000 "
        elem = "11004,11003,12001,12192"
    elif famille in ('radar'):
        FAM = "RADAR"
        VCOTYP = 'Height'
        VCOORD = "  "
        VCOORD = "  "
        STATB = " "
        VCOCRIT = "  "
        elem = "21014"

    elif famille in ('sw', 'sw_qc', 'sw_polar', 'sw_polaireDB'):
        FAM = "AMVS"
        VCOTYP = 'PRESSION'
        VCOORD = " round(vcoord/2000.)*2000 "
        STATB = "OBS_ERROR"
        STATB = " 0."
        VCOCRIT = "  "
        elem = "11002,11004,11003"
    elif famille in ('sf', 'swobnchwos'):
        FAM = "Surface"
        VCOTYP = 'SURFACE'
        VCOORD = " 9999    "
        # VCOORD = " round(lat/30)*30    "
        # VCOORD = "cast(round( (julianday(( (julianday(isodatetime(date,time)) ) )) - julianday(isodatetime(${DATE})))*24.*60./15.,5) as int) +13 "
        VCOCRIT = "  "
        STATB = "OBS_ERROR"
        STATB = "0. "
        elem = "11215,11216,10051,10004,12004,12203,11011,11012 "
    else:
        print("FAMILLE:", famille)
        print("famille no existente. Salida.")
        return None
    return FAM, VCOORD, VCOCRIT, STATB, elem, VCOTYP



import os
import shutil

def delete_create_folder(folder_path, family):
    """
    Deletes a folder and its contents recursively & Creates a new folder at the specified path.

    Args:
        folder_path (str): The path to the folder to be deleted.
        family (str): The name of the subfolder to create within the given path.

    Returns:
        bool: True if the folder was successfully deleted and created, False otherwise.
    """
    try:
        # Construct the full path for the folder
        full_folder_path = os.path.join(folder_path, family)

        # Delete the folder if it exists
        if os.path.exists(full_folder_path):
            shutil.rmtree(full_folder_path)
            print(f"Folder {full_folder_path} deleted successfully along with its contents.")

        # Create the folder
        os.makedirs(full_folder_path)
        print(f"Folder {full_folder_path} created successfully.")

        return True

    except OSError as e:
        print(f"Error: {e}")
        return False



import cartopy.crs as ccrs
import pylab as plt
import numpy as np
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter

def type_projection(Proj):   
   
   """

Projection type
================

 .. image:: ../../../docs/source/_static/proj.png
      :alt: proj Plot


   """

   
   Proj= Proj.lower()
   pc = ccrs.PlateCarree()
   if Proj=='cyl':
    PROJ = ccrs.PlateCarree()
    fig = plt.figure(figsize=(14,7))
    ax = plt.axes(projection=PROJ)
   
    ax.set_extent([-180, 180, -90, 90], crs=PROJ)
    LATPOS=[-90,-60,-30,00,30,60,90]
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)

   elif Proj=='orthon':
    PROJ= ccrs.Orthographic(central_latitude=90.0, central_longitude=-80.0)
    fig = plt.figure(figsize=(14,14))
    ax = plt.axes(projection=PROJ)
   
    ax.set_global()
    ax.coastlines(resolution='110m')
   
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
   elif Proj=='orthos':
    PROJ= ccrs.Orthographic(central_latitude=-90.0, central_longitude=-80.0)
    fig = plt.figure(figsize=(14,14))
    ax = plt.axes(projection=PROJ)
   
    ax.set_global()
    ax.coastlines(resolution='110m')
   
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
    #=================================================================================
   elif Proj=='robinson':
    PROJ = ccrs.Robinson()
    fig = plt.figure(figsize=(15,7))
    ax = plt.subplot(1, 1, 1, projection=PROJ)
   
    ax.set_extent([-180, 180, -90, 90], pc )
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
   # ax.coastlines(resolution='110m')

   elif Proj=='europe':
    PROJ=ccrs.NorthPolarStereo( )#central_longitude=-105.0)
    fig = plt.figure(figsize=[14, 14])
    ax = plt.subplot(1, 1, 1, projection=PROJ)
   
    #ax.set_extent([-50,  53, 42, 90], pc )
    ax.set_extent([-20, 50, 30, 90], crs=ccrs.PlateCarree()) 
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
   
   
   elif Proj=='canada':
  #  PROJ=ccrs.NorthPolarStereo( central_longitude=-105.0)
  #  fig = plt.figure(figsize=[14, 7])
  #  ax = plt.subplot(1, 1, 1, projection=PROJ)
  # 
  #  ax.set_extent([-150,  -53, 42, 90], pc )
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
    PROJ = ccrs.NorthPolarStereo(central_longitude=-105.0)
    fig = plt.figure(figsize=[14, 14])
    ax = plt.subplot(1, 1, 1, projection=PROJ)
   
    # Ajusta el set_extent para la proyección polar
    ax.set_extent([-150, -53, 42, 90], crs=ccrs.PlateCarree())

   elif Proj=='ameriquenord':
    PROJ=ccrs.NorthPolarStereo( central_longitude=-105.0)
    fig = plt.figure(figsize=[14, 7])
    ax = plt.subplot(1, 1, 1, projection=PROJ)
   
    ax.set_extent([-140,  -40, 20, 90], pc )
    ax.set_extent([-170,  -40, 20, 90], pc )
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
   
   elif Proj=='npolar':
    PROJ=ccrs.NorthPolarStereo( central_longitude=-90.5)
    fig = plt.figure(figsize=[13, 7])
    ax = plt.subplot(1, 1, 1, projection=PROJ)
   
    ax.set_extent([-180, 180, 00, 90], pc )
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)
    ax.coastlines(resolution='110m')
   elif Proj=='spolar':
    PROJ=ccrs.SouthPolarStereo( central_longitude=-86)
    fig = plt.figure(figsize=[14, 14])
    ax = plt.subplot(1, 1, 1, projection=PROJ)
   
    ax.set_extent([-180, 180, -90, 00], pc )
    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)

   elif Proj=='hrdps':

    pole_latitude = 35.7
    pole_longitude = 65.5

    PROJ=ccrs.RotatedPole(pole_latitude=pole_latitude, pole_longitude=pole_longitude)
    fig = plt.figure(figsize=[14, 7])
    ax = plt.subplot(1, 1, 1, projection=PROJ)
    lat_0 = 48.8
    delta_lat = 10.
    lon_0 = 266.00
    delta_lon = 40.
    ax.set_extent([lon_0 - delta_lon, lon_0 + delta_lon, lat_0 - delta_lat, lat_0 + delta_lat], pc)


    LATPOS=[-90,-80,-60,-50,-40,-30,00,30,40,50,60,80,90]
    LATPOS=range(-90, 91, 10)




   elif Proj == 'reg':
     Size_mapx=10
     Size_mapy=10
     pole_latitude=31.758
     pole_longitude=178.008 -90.
     llcrnrlat = -7.74911
     llcrnrlon = -128.80635
     urcrnrlat = 57.88194
     urcrnrlon =39.378404
     central_rotated_longitude=-90. -1.991605
     PROJ = ccrs.RotatedPole(pole_longitude=pole_longitude ,  pole_latitude=pole_latitude,
                          central_rotated_longitude=central_rotated_longitude)
     fig=plt.figure(1,figsize=( Size_mapx,Size_mapy))
     ax = fig.add_subplot(1, 1, 1, projection=PROJ)
   
     mapExtent=[llcrnrlon , urcrnrlon,llcrnrlat,urcrnrlat]
     ax.set_extent(mapExtent)
     xs, ys, zs = PROJ.transform_points(pc,
                                    np.array([llcrnrlon, urcrnrlon]),
                                    np.array([llcrnrlat, urcrnrlat])).T
    # print (  ' Xlims=', xs )
   #  print (  ' Ylims=', ys )
   
     ax.set_xlim(xs)
     ax.set_ylim(ys)
     LATPOS=[-60,-30,00,10.,20.,30.,40.,50.,60.,70.,80.,90.]
  
   gl = ax.gridlines(draw_labels=True, linewidth=0.7, color='gray', alpha=0.5, linestyle='--')
   gl.top_labels = False              # Quita etiquetas arriba
   gl.right_labels = False            # Quita etiquetas derecha
   
   # Para dejar los labels bonitos:
   gl.xlabel_style = {'size': 12}
   gl.ylabel_style = {'size': 12}
   gl.xformatter = LongitudeFormatter()
   gl.yformatter = LatitudeFormatter()
   
   # (Opcional) Controlar intervalos:
   gl.xlocator = plt.FixedLocator(np.arange(-180, 181, 30))
   gl.ylocator = plt.FixedLocator(np.arange(-90, 91, 30))
   
   # Si quieres cambiar el texto de los ejes:
   #ax.set_xlabel('Longitud')
   #ax.set_ylabel('Latitud')
   return ax, fig, LATPOS, PROJ, pc
 #  print (  ' HEURE5 Nombre=', datetime.datetime.now().time() )



import os
import sqlite3
import pandas as pd
import matplotlib.pyplot as plt
from glob import glob

# Function to extract date from filename
def extract_date_from_filename(filename, family):
    basename = os.path.basename(filename)
    date_str = basename.split(family)[0][:10]  # 'yyyymmddhh'
    return pd.to_datetime(date_str, format='%Y%m%d%H')


def get_counts_by_date(path, family, start_date_str, end_date_str):
    # Convert to datetime for filtering
    start_date = pd.to_datetime(start_date_str, format='%Y%m%d%H')
    end_date = pd.to_datetime(end_date_str, format='%Y%m%d%H')
    
    # Get all files that match the pattern
    files = glob(os.path.join(path, f'*{family}*'))
    filtered_files = []
    for file in files:
        file_date = extract_date_from_filename(file, family)
        if start_date <= file_date <= end_date:
            filtered_files.append(file)

    date_counts = {}

    for file in filtered_files:
        conn = sqlite3.connect(file)
        cursor = conn.cursor()
        
       # cursor.execute('SELECT COUNT(*) from data  natural join header WHERE id_stn LIKE "NOA%";') 
        cursor.execute('SELECT COUNT(*) from data ;')
      #  cursor.execute('SELECT COUNT(*) FROM data NATURAL JOIN header WHERE id_stn LIKE "NOA%";')
        cursor.execute('SELECT COUNT(*) FROM data JOIN header ON data.id_obs = header.id_obs WHERE header.id_stn LIKE "NOA%";')
        count = cursor.fetchone()[0]
        
        file_date = extract_date_from_filename(file, family)
        
        if file_date not in date_counts:
            date_counts[file_date] = 0
        date_counts[file_date] += count
        
        conn.close()

    return date_counts

# Paths for the two families
#name1, name2 = 'postalt', 'evalalt'
#path1 = f'/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/{name1}/'
#path2 = f'/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/{name2}/'
#name1, name2=  'evalalt', 'dbase'
#path1 = f'/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/{name1}/'
#path2 = f'/home/smco500/.suites/psmon/monitoring/g2/hub/ppp5/sqlite/banco/{name2}/'


combinations = [
    ('postalt', 'evalat', '/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/postalt/', 
                          '/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/evalat/'),
    ('evalalt', 'dbase', '/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/evalalt/', 
                          '/home/smco500/.suites/psmon/monitoring/g2/hub/ppp5/sqlite/banco/dbase/')
]

for name1, name2, path1, path2 in combinations:


   family1, family2 = '*cris*', '*cris*'
   start_date_str = '2024102400'
   end_date_str = '2024102800'
   
   # Colors
   color1, color2 = 'tab:blue', 'tab:green'
   
   # Get counts for both families
   counts_family1 = get_counts_by_date(path1, family1, start_date_str, end_date_str)
   counts_family2 = get_counts_by_date(path2, family2, start_date_str, end_date_str)
   
   # Create DataFrame for easy plotting
   df_family1 = pd.DataFrame(list(counts_family1.items()), columns=['Date', 'Count_family1'])
   df_family2 = pd.DataFrame(list(counts_family2.items()), columns=['Date', 'Count_family2'])
   df_merged = pd.merge(df_family1, df_family2, on='Date', how='outer').fillna(0).sort_values(by='Date')
   
   # Calculate percentage of family2 compared to family1
   df_merged['Percentage'] = (df_merged['Count_family1'] / df_merged['Count_family2']) * 100
   
   # Calculate average percentage
   average_percentage = df_merged['Percentage'].mean()
   
   # Plotting
   fig, ax1 = plt.subplots(figsize=(10, 6))
   
   # Plot percentage on primary y-axis
   ax1.plot(df_merged['Date'], df_merged['Percentage'], marker='o', color='tab:red', label=f'Percentage ({name2} / {name1})')
   ax1.set_xlabel('Date')
   ax1.set_ylabel('Percentage (%)', color='tab:red')
   ax1.tick_params(axis='y', labelcolor='tab:red')
   ax1.set_ylim(0, 105)
   
   # Annotate each point with its percentage
   for i, (date, percentage) in enumerate(zip(df_merged['Date'], df_merged['Percentage'])):
       ax1.text(date, percentage, f'{percentage:.2f}%', ha='center', va='bottom', fontsize=8, color='tab:red')
   
   # Secondary y-axis to show total counts for both families
   ax2 = ax1.twinx()
   ax2.set_ylim(0, 200000000)
   ax2.plot(df_merged['Date'], df_merged['Count_family1'], marker='x', color=color1, label=f'Count {name1}')
   ax2.plot(df_merged['Date'], df_merged['Count_family2'], marker='x', color=color2, label=f'Count {name2}')
   ax2.set_ylabel('Observations (Count)', color='black')
   
   # Adding legends
   ax1.legend(loc='lower left', bbox_to_anchor=(0, 1.15))
   ax2.legend(loc='lower right', bbox_to_anchor=(0, 1.15))
   
   # Display average percentage on the plot
   plt.title(f'Percentage of {name1} over {name2}; Count with Total Observations Over Time\n'
             f'Family: {family1}; Average Percentage: {average_percentage:.2f}%')
   
   # Set grid, rotate x labels, and tight layout
   plt.grid(True)
   plt.xticks(rotation=45)
   plt.tight_layout()
plt.savefig(f'{family1}_{name2}_{name1}_{start_date_str}_{end_date_str}_comparison_plot4.png', dpi=300) 
plt.show()
  



import os
import sqlite3
import pandas as pd
import matplotlib.pyplot as plt
from glob import glob

# Function to extract date from filename
def extract_date_from_filename(filename, family):
    basename = os.path.basename(filename)
    date_str = basename.split(family)[0][:10]  # 'yyyymmddhh'
    return pd.to_datetime(date_str, format='%Y%m%d%H')

def get_counts_by_date(path, family, start_date_str, end_date_str):
    # Convert to datetime for filtering
    start_date = pd.to_datetime(start_date_str, format='%Y%m%d%H')
    end_date = pd.to_datetime(end_date_str, format='%Y%m%d%H')
    
    # Get all files that match the pattern
    files = glob(os.path.join(path, f'*{family}*'))
    filtered_files = []
    for file in files:
        file_date = extract_date_from_filename(file, family)
        if start_date <= file_date <= end_date:
            filtered_files.append(file)

    date_counts = {}

    for file in filtered_files:
        print (file)
        conn = sqlite3.connect(file)
        cursor = conn.cursor()
     #   cursor.execute('CREATE INDEX idx_id_stn ON header (id_stn);')
        # Query to count relevant rows
        cursor.execute('SELECT COUNT(*) FROM data;')#JOIN header ON data.id_obs = header.id_obs WHERE header.id_stn LIKE "NOA%";')
        count = cursor.fetchone()[0]
        
        file_date = extract_date_from_filename(file, family)
        
        if file_date not in date_counts:
            date_counts[file_date] = 0
        date_counts[file_date] += count
        
        conn.close()

    return date_counts

# Paths for the families
families = {
    'postalt': '/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/postalt/',
    'evalat': '/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/evalalt/',
    'dbase': '/home/smco500/.suites/psmon/monitoring/g2/hub/ppp5/sqlite/banco/dbase/'
}

# Dates and pattern for the query
family_pattern = '*cris*'
start_date_str = '2024102400'
end_date_str = '2024102418'

# Dictionary to store counts by date
all_counts = {}

# Loop through each family to get counts
for family, path in families.items():
    counts = get_counts_by_date(path, family_pattern, start_date_str, end_date_str)
    all_counts[family] = counts

# Create DataFrame from counts
df_counts = pd.DataFrame(dict([(k, pd.Series(v)) for k, v in all_counts.items()])).reset_index()
df_counts.rename(columns={'index': 'Date'}, inplace=True)

# Fill missing values with 0
df_counts.fillna(0, inplace=True)

# Calculate percentages
df_counts['Percentage_postalt_evalat'] = (df_counts['postalt'] / df_counts['evalat']) * 100
df_counts['Percentage_evalat_dbase'] = (df_counts['evalat'] / df_counts['dbase']) * 100

# Calculate average percentages
average_percentage_postalt_evalalt = df_counts['Percentage_postalt_evalat'].mean()
average_percentage_evalat_dbase = df_counts['Percentage_evalat_dbase'].mean()
fig, ax1 = plt.subplots(figsize=(10, 6))
ax1.set_ylabel('Percentage (%)', color='tab:red')
ax1.set_xlabel('Date')
ax1.tick_params(axis='y', labelcolor='tab:red')
ax1.set_ylim(0, 105)

ax1.plot(df_counts['Date'], df_counts['Percentage_postalt_evalat'], marker='o', color='tab:red', label=f'Percentage postalt/evalat') 
ax1.plot(df_counts['Date'], df_counts['Percentage_evalat_dbase'], marker='*', color='tab:red', label=f'Percentage evalalt/dbase')

plt.title(f'Count with Total Observations Over Time. Family: {family_pattern};\n'
             f'Average Percentage postalt/evalat : {average_percentage_postalt_evalalt:.2f}% Average Percentage evalalt/dbase : {average_percentage_evalat_dbase:.2f}% ')
   

ax2 = ax1.twinx()
ax2.set_ylabel('Observations (Count)', color='black')
ax2.set_ylim(0, 200000000)
ax2.plot(df_counts['Date'], df_counts['postalt'], marker='x', color='tab:blue' ,label=f'Count postalt')
ax2.plot(df_counts['Date'], df_counts['evalat'], marker='x',  color='tab:green' , label=f'Count evalat')
ax2.plot(df_counts['Date'], df_counts['dbase'], marker='x', color='tab:yellow' ,  label=f'Count dbase')
ax2.set_ylabel('Observations (Count)', color='black')

ax1.legend(loc='lower left', bbox_to_anchor=(0, 1.15))
ax2.legend(loc='lower right', bbox_to_anchor=(0, 1.15))

plt.savefig(f'_plot4.png', dpi=300) 
plt.grid(True)
plt.xticks(rotation=45)

plt.show()


# Print results for debugging
print("Conteos y porcentajes por fecha:")
print(df_counts[['Date', 'postalt', 'evalat', 'dbase', 'Percentage_postalt_evalat', 'Percentage_evalat_dbase']])




import subprocess
import unittest
import os
import sqlite3
import numpy as np
from PIL import Image, ImageChops

def call_pikobs_from_python():
    """
    Calls the pikobs.timeserie.arg_call() function with pre-defined parameters using subprocess.

    Returns:
        tuple: stdout and stderr from the command execution.
    
    Raises:
        RuntimeError: If the command execution fails.
    """
    command = [
        "python", "-c", "import pikobs; pikobs.timeserie.arg_call()",
        "--path_experience_files", "maestro_archives/rel90devg2/monitoring/banco/postalt/",
        "--experience_name", "ops+NOAA21+GOES19",
        "--path_control_files", "maestro_archives/rel90devg2ops/operation.observations.banco.postalt.sqlite.g2/",
        "--control_name", "ops",
        "--pathwork", "onthefly",
        "--datestart", "2025041200",
        "--dateend", "2025041300",
        "--region", "Monde",
        "--family", "sw",
        "--flags_criteria", "assimilee",
        "--fonction", "omp",
        "--id_stn", "all",
        "--channel", "join",
        "--n_cpu", "40"]

    result = subprocess.run(command, capture_output=True, text=True)

    if result.returncode != 0:
        raise RuntimeError(f"The command failed with error code {result.returncode}:\n{result.stderr}")

    return result.stdout, result.stderr

def files_identical(ref_file, gen_file):
    """
    Compare two files of any type based on content.

    Args:
        ref_file (str): Path to the reference file.
        gen_file (str): Path to the generated file.

    Returns:
        bool: True if files are identical, False otherwise.
    """
    if ref_file.endswith('.db'):
        return sqlite_files_are_identical(ref_file, gen_file)

    if ref_file.lower().endswith(('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif')):
        return images_are_identical(ref_file, gen_file)

    return True

def sqlite_files_are_identical(file1, file2):
    """
    Compares two SQLite database files.

    Args:
        file1 (str): Path to the first SQLite file.
        file2 (str): Path to the second SQLite file.

    Returns:
        bool: True if the files are structurally and content-wise identical, False otherwise.
    """
    with sqlite3.connect(file1) as conn1, sqlite3.connect(file2) as conn2:
        cursor1, cursor2 = conn1.cursor(), conn2.cursor()

        # Table and data comparison
        cursor1.execute("SELECT * FROM sqlite_master WHERE type='table';")
        cursor2.execute("SELECT * FROM sqlite_master WHERE type='table';")
        tables1 = set(cursor1.fetchall())
        tables2 = set(cursor2.fetchall())
        if tables1 != tables2:
            print(f"Tables differ between {file1} and {file2}:\n  - {tables1}\n  - {tables2}")
            return False

        for table in tables1:
            print(f"Comparing table {table[1]}")
            cursor1.execute(f"SELECT * FROM {table[1]} ORDER BY 1;")
            rows1 = sorted(cursor1.fetchall())

            cursor2.execute(f"SELECT * FROM {table[1]} ORDER BY 1;")
            rows2 = sorted(cursor2.fetchall())
            if rows1 != rows2 or not np.array_equal(rows1, rows2):
                print(f"Row data differs in table {table[0]}")
                return False

    return True

def images_are_identical(img1_path, img2_path):
    """
    Check if two images are identical.

    Args:
        img1_path (str): Path to the first image.
        img2_path (str): Path to the second image.

    Returns:
        bool: True if images are identical, False otherwise.
    """
    img1 = Image.open(img1_path)
    img2 = Image.open(img2_path)

    if img1.size != img2.size or img1.mode != img2.mode:
        return False

    diff = ImageChops.difference(img1, img2)
    return diff.getbbox() is None

def files_exist_and_are_identical(reference_dir, generated_dir):
    """
    Check if all reference files exist in the generated directory and match in content.

    Args:
        reference_dir (str): Path to the directory containing reference files.
        generated_dir (str): Path to the directory containing generated files.

    Returns:
        bool: True if all files exist and are identical, False otherwise.
    """
    for root, _, files in os.walk(reference_dir):
        for name in files:
            ref_file_path = os.path.join(root, name)
            rel_path = os.path.relpath(ref_file_path, reference_dir)
            gen_file_path = os.path.join(generated_dir, rel_path)
            if not os.path.exists(gen_file_path) or not files_identical(ref_file_path, gen_file_path):
                return False

    return True

class TestCallPikObs_timeserie(unittest.TestCase):
    """
    Unit test suite for verifying file generation and comparison.

    Attributes:
        path_reference (str): Path to the reference directory

    Methods:
        setUp(): Prepares the test environment.
        test_call_and_compare(): Tests the call to pikobs and file comparisons.
    """
    path_reference = None

    def setUp(self):
        self.path_reference = self.__class__.path_reference

    def test_call_and_compare(self):
        if not self.path_reference:
            self.fail("Path to the reference directory must be provided.")

        try:
            call_pikobs_from_python()

            path_to_validate = os.path.abspath('onthefly')
            if not os.path.exists(self.path_reference):
                self.fail(f"Reference directory does not exist: {self.path_reference}")

            self.assertTrue(
                files_exist_and_are_identical(self.path_reference, path_to_validate),
                "Not all reference files exist in the generated directory with identical content."
            )
        except RuntimeError as e:
            self.fail(f"call_pikobs_from_python raised an exception: {e}")



def flag_criteria(flags):

 
    """
Flag Criteria for Filtering Elements:
=====================================

In data processing and analysis, flag criteria are essential for selecting and managing elements based on specific conditions or quality assessments. These criteria allow users to filter data effectively, focusing on elements that meet predefined standards or have undergone particular processing stages.

Possible flag Criteria:
=======================

- **--flag_criteria all** (all elements): Selects all available elements without applying restrictions based on specific bits.
- **--flag_criteria** assimilee` (assimilated elements): Selects elements that have the BIT12 (4096) active, indicating that the element has influenced the analysis.
- **--flag_criteria** bgckalt` (approved by Background Check): Excludes elements that have the BIT9 (512) and BIT11 (2048) active, indicating rejections due to AO quality control and the selection process (thinning or canal), respectively.
- **--flag_criteria** bgckalt_qc` (approved by Background Check and QC-Var): Similar to `bgckalt`, but also excludes elements with BIT9 (512) and BIT11 (2048) active.
- **--flag_criteria** monitoring` (monitoring): This filter excludes elements with BIT9 (512) and BIT7 (128) active, indicating rejection by AO quality control (Background Check or QC-Var) and being in reserve, respectively.
- **--flag_criteria** postalt` (postal): This filter excludes elements with BIT17 (131072), BIT9 (512), BIT11 (2048), and BIT8 (256) active, indicating rejection by QC-Var, AO quality control, the selection process (thinning or canal), and being in the blacklist, respectively.

Description of Active Bits:
===========================

- **BIT0** (1) - Modified or generated by ADE.
- **BIT1** (2) - Element that exceeds a climatological extreme or fails consistency test.
- **BIT2** (4) - Erroneous element.
- **BIT3** (8) - Potentially erroneous element.
- **BIT4** (16) - Dubious element.
- **BIT5** (32) - Interpolated element, generated by DERIVATE.
- **BIT6** (64) - Corrected by DERIVATE sequence or bias correction.
- **BIT7** (128) - Rejected by satellite QC.
- **BIT8** (256) - Element rejected because it is on a blacklist.
- **BIT9** (512) - Element rejected by AO quality control (Background Check or QC-Var).
- **BIT10** (1024) - Generated by AO.
- **BIT11** (2048) - Rejected by a selection process (thinning or canal).
- **BIT12** (4096) - Assimilated.
- **BIT13** (8192) - Comparison against test field, level 1.
- **BIT14** (16384) - Comparison against test field, level 2.
- **BIT15** (32768) - Comparison against test field, level 3.
- **BIT16** (65536) - Rejected by comparison against test field (Background Check).
- **BIT17** (131072) - Rejected by QC-Var.
- **BIT18** (262144) - Not used due to orography.
- **BIT19** (524288) - Not used due to land-sea mask.
- **BIT20** (1048576) - Aircraft position error detected by TrackQc.
- **BIT21** (2097152) - Inconsistency detected by a CQ process.
- **BIT22** (4194304) - Rejected due to transmittance above model top
- **BIT23** (8388608) - Cloud affected radiance
    """

    # Define bit masks for flag criteria
    BIT6_BIASCORR = 64  
    BIT7_REJ = 128  
    BIT8_BLACKLIST = 256
    BIT9_REJ = 512
    BIT11_SELCOR = 2048  
    BIT12_VUE = 4096 
    BIT16_REJBGCK = 65536
    BIT17_QCVAR = 131072

    # Generate filtering criteria based on the selected flag
    if flags == "all":
        FLAG = "flag >= 0"
    elif flags == "assimilee":
        FLAG = f"and (flag & {BIT12_VUE}) = {BIT12_VUE}"
    elif flags == "bgckalt":
        FLAG = f"and (flag & {BIT9_REJ}) = 0 and (flag & {BIT11_SELCOR}) = 0 and (flag & {BIT8_BLACKLIST}) = 0"
    elif flags == "bgckalt_qc":
        FLAG = f"and (flag & {BIT9_REJ}) = 0 and (flag & {BIT11_SELCOR}) = 0"
    elif flags == "monitoring":
        FLAG = f"and (flag & {BIT9_REJ}) = 0 and (flag & {BIT7_REJ}) = 0"
    elif flags == "postalt":
        FLAG = f"and (flag & {BIT17_QCVAR}) = 0 and (flag & {BIT9_REJ}) = 0 and (flag & {BIT11_SELCOR}) = 0 and (flag & {BIT8_BLACKLIST}) = 0"
    elif flags == "rejets_qc":
        FLAG = f"and (flag & {BIT9_REJ}) = {BIT9_REJ}"
    elif flags == "rejets_bgck":
        FLAG = f"and (flag & {BIT16_REJBGCK}) = {BIT16_REJBGCK}"
    elif flags == "bias_corr":
        FLAG = f"and (flag & {BIT6_BIASCORR}) = {BIT6_BIASCORR}"
    elif flags == "qc":
        FLAG = f"and (flag & {BIT9_REJ}) = 0"
    else:
        raise ValueError(f'Invalid flag option: {flags}')

    return FLAG



#!/usr/bin/python3
import numpy as np
import sys
import csv
import dateutil
import string
import math
import matplotlib as mpl
mpl.use('Agg')
import pylab as plt
import sqlite3
from matplotlib import ticker, font_manager, dates
import matplotlib
from datetime import date
import pikobs
# -----------------------------------------------------------------------------------------------------------------
# VERSION 2:  Added  ${qc_flags} argument.
#             Allow for plots with missing O-A (monitoring, bgckalt) and missing BCor
# -----------------------------------------------------------------------------------------------------------------

# USAGE:                        1         2          3         4         5
# > satellite_cardio_plot.py ${file}  ${channel}  ${TITLE}  ${type}  ${qc_flags}

# channel = "10"              plot channel 10 out of multiple channels in file
#         = "0"               single channel or channels combined  (channel numbers are NOT in file)
#                             (Channel info will be in TITLE)

# e.g. > satellite_cardio_plot_v2.py /data/cmda/afsd/abe/sortir_les_resultats/cardiogrammes/burp2rdb_plots_v2/G2_50BE14AB4/serie_ssmis_Monde_G2_50BE14AB4_DMSP16 '12' 'G2_50BE14AB4  Monde  DMSP16-ssmis Ch.12' wide assimilee


# Process arguments (into strings)
def cardio_plot(pathwork,datestart, dateend, flag_type, family, fig_title, plot_type, channel, id_stn, varno, region):
  
 # print ('PINTAR')
  LARG=[]
  matplotlib.pyplot.close()
  GREEN='#009900'
  BLUE='#0276FD'
  BLACK='#000000'
  RED='#D00000'
  
  MISSING=-99.9
  
 # print (' LARG=',LARG)
  

  #datfile   = LARG[1]
 # channel   = LARG[2]
  #ichannel = int(channel)
  #fig_title = LARG[3]
  #plot_type = LARG[4]  #  classic or wide
  #flag_type = LARG[5]  #  monitoring bgckalt postalt assimilee all
  

  if plot_type == 'classic':
    #            w    h  (inches)
    fig_size = (8.5,11.0)
  elif plot_type == 'wide':
    fig_size = (15.0,10.0)
  else:
    print ("Invalid plot type =",plot_type)
    sys.exit()
  
  if flag_type == 'monitoring' or flag_type == 'bgckalt' or flag_type == 'bgckalt_qc':
    is_oma = False
  else:
    is_oma = True
  
  
  #                 Input CSV file with stats results grouped by channel
  #
  #       0             1    2    3     4      5      6       7     8      9       10     11       12
  #   DATE            ,Chan,Nrej,Nacc,AvgOMP,AvgOMA,StdOMP,StdOMA,NDATA,Nprofils,AvgBCOR,AvgOBS,  Ntot
  #2011-01-24 00:00:00, 12,  0,  381, 0.5461, 0.298,2.1832,1.2944,381,  381,     0.1669, 133.186, 1621
  #2011-01-24 00:00:00 ,13,  0,  396, 0.208, 0.0207,1.3537,0.7966,396,  396,    -3.0804,195.3015, 1621
  #.....
  #         Input file with stats results NOT grouped by channel (channel combination)
  #
  #       0               1     2    3      4      5      6      7     8         9       10     12
  #   DATE            , Nrej,Nacc,AvgOMP,AvgOMA,StdOMP,StdOMA,NDATA,Nprofils, AvgBCOR,AvgOBS,  Ntot
  #2011-01-24 00:00:00,  0,  381, 0.5461, 0.298,2.1832,1.2944,381,   381,     0.1669, 133.186, 1621
  #2011-01-24 06:00:00,  0,4105,  0.5088,0.3161,2.2415,1.2529,4105, 4105,     0.2958,134.2117, 18279
  #.....
  
  # Ntot = number of data locations in HEADER of SQLite rdb file
  
  
  # Set timer on and pragma TEMP_STORE=memory
  path_output_work=f'{pathwork}/{family}/cardio_{region}_{datestart}_{dateend}_{flag_type}_{family}.db'
 # print (path_output_work) 
  conn = sqlite3.connect(path_output_work)
  # Create a cursor to execute SQL queries
  #FNAM, FNAMP, SUM, SUM2 = pikobs.type_boxes(fonction)
  cursor = conn.cursor()

  cursor.execute("PRAGMA TEMP_STORE=memory")
  critery_stn=f"AND id_stn='{id_stn}'"
  critery_chan=f"and Chan='{channel}'"

  if id_stn=='join':
       critery_stn=' '
  if channel=='join':
       critery_channel=' '
  query = f"""
           SELECT
                DATE,  
                Chan,
                Nrej,
                Nacc,
                AvgOMA,
                AvgOMP,
                StdOMA,
                StdOMP,
                AvgOBS,
                NDATA,
                Nprofile,
                Ntot,
                AvgBCOR
            FROM
               serie_cardio
            WHERE
                varno='{varno}'
                {critery_stn}
                {critery_chan}
                 
            GROUP BY
                DATE;"""
  cursor.execute(query)
  results = cursor.fetchall() 
  Dates  = np.array([row[0] for row in results])  #Dates
  Chans   = np.array([row[1] for row in results])  #Chans
  aoma    = np.array([row[4] for row in results])  #AvgOMA
  aomp    = np.array([row[5] for row in results])  #AvgOMP
  StdOMA  = np.array([row[6] for row in results])  #StdOMA
  StdOMP  = np.array([row[7] for row in results])  #StdOMP
  aobs    = np.array([row[8] for row in results])  #AvgOBS
  NDATA   = np.array([row[9] for row in results]) #NData
  Ntot    = np.array([row[11] for row in results]) #Ntot
  abcor   = np.array([row[12] for row in results]) #AvgBCOR
  #sys.exit()
  #print (Dates, Chans )
  #cursor.execute(". nullvalue -99.9")
#  exit()
#  sys.exit()
#  Dates=[]
#  Chans=[]
#  AvgOMP=[]
#  AvgOMA=[]
#  StdOMP=[]
#  StdOMA=[]
#  NData=[]
#  AvgBCOR=[]
#  AvgOBS=[]
  
  L1=[]  # List containing all rows (lists) of data read from csv file
  rownum1 = 0
  
  #----------------------------------------------------------------------
  # Read data from input CSV file
  #   row = list containing column data (as strings) for a row
  #----------------------------------------------------------------------
  
  #hdl1 = open (datfile,'r')                  # Open file for reading only
  #reader1 = csv.reader(hdl1,delimiter=",")   # Read file, skipping first (header) line
  #for row in reader1:
  #   if rownum1 != 0:
  #      if ichannel != 0:                    # Only select rows for specified channel!
  #         ichan = int(row[header.index("Chan")])
  #         if ichan == ichannel:
  #            L1.append( row )
  #      else:
  #         L1.append( row )
  #   else:
  #      header = row
  #      print ("Header from file =",header)
  #   rownum1 += 1
  
  #L1.sort(key=lambda tup: tup[0])
  
  # Extract the time series (columns) for plotting from data rows read
  # Convert the string values to appropriate data type
  
  #if "DATE" in header:
  #  Dates   = [da[header.index("DATE")] for da in L1]
  #  pdates  = [dateutil.parser.parse(s) for s in Dates]
  #else:
  #  print ("DATE not found in header!")
  #  sys.exit()
  #
  #if "AvgOMP" in header:
  #  AvgOMP  = [x[header.index("AvgOMP")]  for x in L1]

  #  AvgOMP  = [float(x) for x in AvgOMP]
  #  aomp    = np.array(AvgOMP)
  #else:/
  #  print ("AvgOMP not found in header!")
  #  sys.exit()
  # 
  #if "AvgOMA" in header:
  #  AvgOMA  = [x[header.index("AvgOMA")]  for x in L1]
  #  AvgOMA  = [float(x) for x in AvgOMA]
  #  aoma    = np.array(AvgOMA)
  #else:
  #  print ("AvgOMA not found in header!")
  #  sys.exit()
 # 
  #if "StdOMP" in header:
   # StdOMP  = [x[header.index("StdOMP")]  for x in L1]
  #  StdOMP  = [float(x) for x in StdOMP]
  #else:
  #  print ("StdOMP not found in header!")
 #   sys.exit()
  
 # if "StdOMA" in header:
 #   StdOMA  = [x[header.index("StdOMA")]  for x in L1]
 ##   StdOMA  = [float(x) for x in StdOMA]
 # else:
  #  print ("StdOMA not found in header!")
  #  sys.exit()
  
  #if "NDATA" in header:
  #  NData   = [x[header.index("NDATA")]   for x in L1]
  #  NData   = [int(x) for x in NData]
 # else:
 #   print ("NDATA not found in header!")
 #   sys.exit()
  
 # if "AvgBCOR" in header:
 #   AvgBCOR = [x[header.index("AvgBCOR")] for x in L1]
 #   AvgBCOR = [float(x) for x in AvgBCOR]
  #  abcor   = np.array(AvgBCOR)
 # else:
 #   print ("AvgBCOR not found in header!")
 #   sys.exit()
  
 # if "AvgOBS" in header:
  #  AvgOBS  = [x[header.index("AvgOBS")] for x in L1]
  #  AvgOBS  = [float(x) for x in AvgOBS]
  #  aobs    = np.array(AvgOBS)
  #else:
  #  print ("AvgOBS not found in header!")
  #  sys.exit()
  
  #if "Ntot" in header:
  #  Ntot   = [x[header.index("Ntot")]   for x in L1]
  #  Ntot   = [int(x) for x in Ntot]
  #else:
  #  print ("Ntot not found in header!")
  #  sys.exit()
  
 # if ichannel != 0:
  #  Chans  = [x[header.index("Chan")] for x in L1]  # list should contain selected channel number only
  #  Chans  = [int(x) for x in Chans]
  
  # Check if AvgOMA, StdOMA are missing (=MISSING)
  
  oma = list(set(aoma))
  if len(oma) == 1 and oma[0] == MISSING:
    if is_oma:
      print ("OMA data are missing! Aborting....")
      sys.exit()
    else:
      is_oma = False
  else:
    is_oma = True
  
  # Check if AvgBCOR are missing; if yes then set values to 0
  
  bc = list(set(abcor))
  is_bcor = True
  if len(bc) == 1 and bc[0] == MISSING:
    if flag_type != 'monitoring':
      print ("WARNING: AvgBCOR data are missing.")
    AvgBCOR[:] = 0.0
    abcor = np.array(AvgBCOR)
    is_bcor = False
  try:
     raw_omp = aomp - abcor
  except:
     raw_omp = aomp  
  try:
     raw_obs = aobs - abcor
  except:
     raw_obs = aobs 
  p = aobs - aomp
  if is_oma:
    a = aobs - aoma
  N  = NDATA
  NT = Ntot
  
  Bias1  = aomp
  Sigma1 = StdOMP
  Mu  = sum(Bias1*N)/sum(N)
  Sx2 = (Sigma1*Sigma1 +Bias1*Bias1)*N
  Sig = math.sqrt(sum(Sx2)/sum(N)  -Mu*Mu)
  Mean_OMP = Mu
  Mean_raw_OMP = sum(raw_omp*N)/sum(N)
  Std_OMP  = Sig
  STRING_OMP = 'O-P Mean: '+str(round(Mu,3)) + ' Sigma: '+str(round(Sig,3)) + ' Ndata: '+ str( int(sum(N)))
  
  if is_oma:
    Bias2  = aoma
    Sigma2 = StdOMA
    Mu  = sum(Bias2*N)/sum(N)
    Mean_OMA = Mu
    Sx2 = (Sigma2*Sigma2 +Bias2*Bias2)*N
    Sig = math.sqrt(sum(Sx2)/sum(N)  -Mu*Mu)
    Std_OMA  = Sig
    STRING_OMA = 'O-A Mean: '+str(round(Mu,3)) + ' Sigma: '+str(round(Sig,3)) + ' Ndata: '+ str( int(sum(N)))
  
  try:
    Avg_BCOR = sum(abcor*N)/sum(N)
  except:
    Avg_BCOR = 0
  Avg_N = float(sum(N))/len(N)
  Avg_NT = float(sum(NT))/len(NT)
  
  debut=Dates[0]
  fin  =Dates[-1]
  PERIODE='From ' + debut +'   to ' + fin
  TITLE=fig_title
  
  MEAN_TITLE1 = str('Mean(O-P) = '+str(round(Mean_OMP,3))) + '  Raw Mean(O-P) = '+str(round(Mean_raw_OMP,3)) # red
  if is_oma:
    MEAN_TITLE2 = str('Mean(O-A) = '+str(round(Mean_OMA,3)))  # blue
  else:
    MEAN_TITLE2 = ''
  if is_bcor:
    MEAN_TITLE3 = str('Mean(Bcor) = '+str(round(Avg_BCOR,3))) # green
  else:
    MEAN_TITLE3 = ''
  
  STD_TITLE1  = str('Std(O-P) = '+str(round(Std_OMP,3)))    # red
  if is_oma:
    STD_TITLE2  = str('Std(O-A) = '+str(round(Std_OMA,3)))    # blue
  else:
    STD_TITLE2  = ''
  
  #----------------------------------------------------------------------
  #  Plot Section:      4 plots on 1 page (fig) using plt.axes()
  #----------------------------------------------------------------------
  
  left = 0.1   # left margin
  width = 0.8  # width of the plots
  
  fp=font_manager.FontProperties(size='small')  # set font props for legends
  
  fig = plt.figure(figsize=fig_size,facecolor='white')
  
  p1_bottom = 0.73
  p2_bottom = 0.51
  p3_bottom = 0.29
  p4_bottom = 0.07
  height  = 0.18
  
  # Position the 4 plots on the page (figure)
  ax1 = plt.axes([left, p1_bottom, width, height])
  ax2 = plt.axes([left, p2_bottom, width, height])
  ax3 = plt.axes([left, p3_bottom, width, height])
  ax4 = plt.axes([left, p4_bottom, width, height])
  
  # hide grid lines
  ax1.grid(False)
  ax2.grid(False)
  ax3.grid(False)
  ax4.grid(False)
  
  # ticks
  #ax1.xaxis.tick_bottom()
  #ax2.xaxis.tick_bottom()
  #ax3.xaxis.tick_bottom()
  #ax4.xaxis.tick_bottom()
  
  # X-axis values (dates) for plots
  variable_name, unit, vcoord_type = pikobs.type_varno(f"{varno}")
  def format_date(x, pos=None):
      return plt.num2date(x).strftime('%b %d:%HZ')
  
  ax1.xaxis.set_major_formatter(ticker.FuncFormatter(format_date))
  ax2.xaxis.set_major_formatter(ticker.FuncFormatter(format_date))
  ax3.xaxis.set_major_formatter(ticker.FuncFormatter(format_date))
  ax4.xaxis.set_major_formatter(ticker.FuncFormatter(format_date))
  
  x = plt.date2num(Dates)
  xmi=x[0]
  xma=x[-1]
  #ax1.set_xlim([xmi, xma])
  #ax2.set_xlim([xmi, xma])
  #ax3.set_xlim([xmi, xma])
  #ax4.set_xlim([xmi, xma])
  ax1.plot(x, aomp, linestyle='-',  color=RED,lw=2,   label='O-P')
  ncols = 1
  if is_oma:
    ax1.plot(x, aoma, linestyle='-',  color=BLUE,lw=2,  label='O-A')
    ncols += 1
  if is_bcor:
    ax1.plot(x, abcor, linestyle='-', color=GREEN,lw=2, label='bcor')
    ax1.plot(x, raw_omp, linestyle=':',color=RED,lw=2,   label='(O-P)raw')
    ncols += 2
  ax1.set_xlim([xmi, xma])
  ax1.axhline(color='purple',lw=2)
  ax1.set_ylabel(f'Mean {unit}')
  #ax1.legend(loc=
  #  best          0
  #  upper right   1
  #  upper left    2
  #  lower left    3
  #  lower right   4
  ax1.legend(loc=2,ncol=ncols,prop=fp)
  ax1.yaxis.set_minor_locator(ticker.AutoMinorLocator())
  ax1.xaxis.set_minor_locator(dates.DayLocator())
  ax1.yaxis.grid(True, which='major', linestyle='-', color='grey')
  ax1.yaxis.grid(True, which='minor', linestyle=':', color='grey')
  ax1.xaxis.grid(True, which='major', linestyle=':', color='grey')
  ax1.xaxis.grid(True, which='minor', linestyle=':', color='grey')
  ax1.text(0.0,  1.0, MEAN_TITLE1, horizontalalignment='left', verticalalignment='bottom', transform=ax1.transAxes, color=RED,fontsize=11)
  ax1.text(0.52, 1.0, MEAN_TITLE2, horizontalalignment='left', verticalalignment='bottom', transform=ax1.transAxes, color=BLUE, fontsize=11)
  ax1.text(0.77, 1.0, MEAN_TITLE3, horizontalalignment='left', verticalalignment='bottom', transform=ax1.transAxes, color=GREEN,fontsize=11)
  for label in ax1.get_xticklabels():
        label.set_fontsize(8)
   # extend y-axis range by 1 major tick interval so legend does not cover data plot
  yt=ax1.yaxis.get_majorticklocs() # numpy array with locations of y-ticks in y-axis units (with extra ticks at each end)
  ylims=ax1.get_ylim()
  if yt[-1] == ylims[1]:
    delt = yt[-1] - yt[-2]
    ylim2 = yt[-1] + delt
    ax1.set_ylim([ylims[0],ylim2])
  else:
    ax1.set_ylim([ylims[0],yt[-1]])
  
  ax2.plot(x, StdOMP, linestyle='-', color=RED,lw=2, label='O-P')
  ncols = 1
  if is_oma:
    ax2.plot(x, StdOMA, linestyle='-', color=BLUE,lw=2, label='O-A')
    ncols += 1
  ax2.set_xlim([xmi, xma])
  ylims = ax2.get_ylim()
  if is_oma:
    ax2.set_ylim([0,ylims[1]])
  ax2.set_ylabel(f'StdDev {unit}')
#  ax2.set_ylim(0, 5)
  ax2.legend(loc=3,ncol=ncols,prop=fp)
  ax2.yaxis.set_minor_locator(ticker.AutoMinorLocator())
  ax2.xaxis.set_minor_locator(dates.DayLocator())
  ax2.yaxis.grid(True, which='major', linestyle='-', color='grey')
  ax2.yaxis.grid(True, which='minor', linestyle=':', color='grey')
  ax2.xaxis.grid(True, which='major', linestyle=':', color='grey')
  ax2.xaxis.grid(True, which='minor', linestyle=':', color='grey')
  ax2.text(0.0,   1.0, STD_TITLE1, horizontalalignment='left', verticalalignment='bottom', transform=ax2.transAxes, color=RED,fontsize=11)
  ax2.text(0.25,  1.0, STD_TITLE2, horizontalalignment='left', verticalalignment='bottom', transform=ax2.transAxes, color=BLUE, fontsize=11)
  for label in ax2.get_xticklabels():
        label.set_fontsize(8)
  
  ax3.plot(x, aobs, linestyle='-', color=BLACK, label='Obs')
  ncols = 1
  if is_bcor:
    ax3.plot(x, raw_obs, linestyle=':', color=BLACK, label='RawObs')
    ncols += 1
  ax3.plot(x, p, color=RED, label='Trial')
  ncols += 1
  if is_oma:
    ax3.plot(x, a, color=BLUE, label='Anal')
    ncols += 1
  ax3.set_xlim([xmi, xma])
  ax3.set_ylabel(f'{variable_name} {unit}')
  ax3.legend(loc=0,ncol=ncols,prop=fp)
  ax3.yaxis.set_minor_locator(ticker.AutoMinorLocator())
  ax3.xaxis.set_minor_locator(dates.DayLocator())
  ax3.xaxis.grid(True, which='major', linestyle=':', color='grey')
  ax3.xaxis.grid(True, which='minor', linestyle=':', color='grey')
  for label in ax3.get_xticklabels():
        label.set_fontsize(8)
  
  ax4.plot(x, NDATA, linestyle='-', color=BLACK, label='Ndata' )
  ax4.plot(x, Ntot,  linestyle=':', color=BLACK, label='Nlocs')
  ax4.set_xlim([xmi, xma])
  ylims = ax4.get_ylim()
  ax4.set_ylim([0,ylims[1]])
  ax4.set_ylabel('Number of Observations')
  ax4.legend(loc=0,ncol=2,prop=fp)
  ax4.yaxis.set_minor_locator(ticker.AutoMinorLocator())
  ax4.xaxis.set_minor_locator(dates.DayLocator())
  ax4.xaxis.grid(True, which='major', linestyle=':', color='grey')
  ax4.xaxis.grid(True, which='minor', linestyle=':', color='grey')
  for label in ax4.get_xticklabels():
        label.set_fontsize(8)
  title4="Average number of data = "+str(int(round(Avg_N)))+"  Avg Nlocs = "+str(int(round(Avg_NT)))
  ax4.set_title(title4)
  
  
  fig.autofmt_xdate()
  fig.suptitle(f'Experience:{TITLE} Satellite:{id_stn} channel:{channel} ({region})', fontsize=15, fontweight='bold')
  
  #plt.show()

  plt.savefig(f'{pathwork}/{family}/serie_{TITLE}_{family}_{id_stn}_id_stn_{region}_ch{channel}_varno{varno}.png',format='png',dpi=100)
  plt.close()



import sqlite3
import glob
import os

import sqlite3

def create_and_insert_flag_observations(input_file, output_file):
    # Connect to the input SQLite file
    conn_input = sqlite3.connect(input_file)
    cursor_input = conn_input.cursor()
    print (input_file)
    # Connect to the output SQLite file
    conn_output = sqlite3.connect(output_file)
    cursor_output = conn_output.cursor()

    # Create a table in the output database if it does not exist
    cursor_output.execute('''
        CREATE TABLE IF NOT EXISTS flag_observations (
            date TEXT,
            time TEXT,
            flag INTEGER,
            num_data_entries INTEGER,
            id_stn INTEGER
        )
    ''')

    # Query to get the required data
    query = '''
        SELECT 
            h.date, 
            h.time, 
            d.flag, 
            COUNT(*) AS num_data_entries, 
            h.id_stn
        FROM 
            data d
        JOIN 
            header h ON d.id_obs = h.id_obs
        GROUP BY 
            h.date, 
            h.time, 
            d.flag, 
            h.id_stn
        ORDER BY 
            h.date, 
            h.time, 
            h.id_stn, 
            d.flag;
    '''
    
    # Execute the query
    cursor_input.execute(query)
    results = cursor_input.fetchall()

    # Insert results into the output database
    for date, time, flag, num_data_entries, id_stn in results:
        cursor_output.execute('''
            INSERT INTO flag_observations (date, time, flag, num_data_entries, id_stn)
            VALUES (?, ?, ?, ?, ?)
        ''', (date, time, flag, num_data_entries, id_stn))

    # Close the connections
    conn_input.close()
    conn_output.commit()
    conn_output.close()

# Usage of the script

# Specify the directory path
directory = '/home/sprj700/data_maestro/ppp6/maestro_archives/G2FC900V2E22/monitoring/banco/postalt/'

# Use glob to get the list of files in the directory
print (directory)
file_list = glob.glob(os.path.join(directory, '20220812*amsua_allsky'))
output_file="output_file.db"
for filein in file_list:
   create_and_insert_flag_observations(filein, output_file)



#!/usr/bin/env python3

#Auteur : Pierre Koclas, May 2021
import os
import sys
import csv
from math import floor,ceil,sqrt
import matplotlib as mpl
mpl.use('Agg')
#import pylab as plt
import matplotlib.pylab as plt
import numpy as np
import matplotlib.colorbar as cbar
import matplotlib.cm as cm
import datetime
import cartopy.crs as ccrs
import cartopy.feature
#from cartopy.mpl.ticker    import LongitudeFormatter,  LatitudeFormatter
import matplotlib.colors as colors
#import matplotlib.patches as mpatches
from matplotlib.colors import ListedColormap, LinearSegmentedColormap
import sqlite3
from matplotlib.collections import PatchCollection
from statistics import median
import pikobs
import optparse
from mpl_toolkits.axes_grid1 import make_axes_locatable
def projectPpoly(PROJ,lat,lon,deltax,deltay,pc):
        X1,Y1  = PROJ.transform_point(lon - deltax,lat-deltay,pc )
        X2,Y2  = PROJ.transform_point(lon - deltax,lat+deltay,pc )
        X3,Y3  = PROJ.transform_point(lon + deltax,lat+deltay,pc )
        X4, Y4 = PROJ.transform_point(lon + deltax,lat-deltay,pc )
        Pt1=[ X1,Y1 ]
        Pt2=[ X2,Y2 ]
        Pt3=[ X3,Y3 ]
        Pt4=[ X4,Y4 ]
        Points4 = [ Pt1, Pt2,Pt3,Pt4 ]
           
        return Points4
def SURFLL(lat1,lat2,lon1,lon2):
#= (pi/180)R^2 |sin(lat1)-sin(lat2)| |lon1-lon2|
    R=6371.
    lat2=min(lat2,90.)
    surf=R*R*(np.pi/180.)*abs ( np.sin(lat2*np.pi/180.) - np.sin(lat1*np.pi/180.) ) *abs( lon2-lon1 )
   # if ( surf == 0.):
    # print (   ' surf=',lat1,lat2,lat2*np.pi/180.,lat1*np.pi/180.,np.sin(lat2*np.pi/180.) ,  np.sin(lat1*np.pi/180.) )
    return surf

def NPSURFLL(lat1, lat2, lon1, lon2):
    R = 6371.
    lat2 = np.minimum(lat2, 90.)
    surf = R**2 * (np.pi/180) * np.abs(np.sin(lat2*np.pi/180) - np.sin(lat1*np.pi/180)) * np.abs(lon2 - lon1)
  #  if np.any(surf == 0.):
    #    print('surf contiene valores cero')
    return surf
def SURFLL2(lat1, lat2, lon1, lon2):
    R = 6371.0
    lat2 = np.minimum(lat2, 90.0)
    surf = R * R * (np.pi / 180.0) * np.abs(np.sin(lat2 * np.pi / 180.0) - np.sin(lat1 * np.pi / 180.0)) * np.abs(lon2 - lon1)
    # Debugging print statements if surface is zero
    zero_surf_indices = (surf == 0.0)
    if np.any(zero_surf_indices):
        print('surf=', lat1[zero_surf_indices], lat2[zero_surf_indices], lat2[zero_surf_indices] * np.pi / 180.0,
              lat1[zero_surf_indices] * np.pi / 180.0,
              np.sin(lat2[zero_surf_indices] * np.pi / 180.0),
              np.sin(lat1[zero_surf_indices] * np.pi / 180.0))
    return surf
import pikobs
import pikobs

import pikobs

import pikobs
import pikobs

import pikobs

def zone_ploat(
    mode,
    region,
    family,
    id_stn,
    datestart,
    dateend,
    Points,
    boxsizex,
    boxsizey,
    proj,
    pathwork,
    flag_criteria,
    fonction,
    vcoord,
    filesin,
    namesin,
    varno,
    intervales
):
    import pandas as pd
    import matplotlib.pyplot as plt
    from matplotlib.patches import Rectangle
    from matplotlib.collections import PatchCollection
    from matplotlib import cm, colors, colorbar
    import numpy as np
    import sqlalchemy
    import os
    from matplotlib.colors import ListedColormap, BoundaryNorm
    print ('s')
    method = 1
    vcoord_type = 'Channel'
    if family=='sw':
      vcoord_type = 'Pressure(Hpa)'

    debut = datestart
    final = dateend
    sqlite_files = filesin
    os.makedirs(pathwork, exist_ok=True)

    def load_data(sqlite_file):
        engine = sqlalchemy.create_engine(f"sqlite:///{sqlite_file}")
        query = f"""
        SELECT
            id_stn, varno,
            vcoord AS vcoord_bin,
            round(lat/1.0)*1.0 AS lat_bin,
            SUM(sumy)*1.0/SUM(N) AS omp,
            SQRT(SUM(sumy2)*1.0/SUM(N) - POWER(SUM(sumy)*1.0/SUM(N), 2)) AS sigma,
            SUM(N) AS n_obs
        FROM moyenne
        WHERE
            varno = {varno}
            AND id_stn = '{id_stn}'
            AND sumy IS NOT NULL
            AND date BETWEEN '{debut}' AND '{final}'
        GROUP BY id_stn, varno, vcoord_bin, lat_bin
        HAVING COUNT(*) > 1
        ORDER BY vcoord_bin DESC, lat_bin
        """
        return pd.read_sql_query(query, engine)

    # -- FUNCION PARA CREAR UNA COLORBAR DIVERGENTE CON GRIS EN 0 --
    def custom_div_cbar(bounds, cmap_name='seismic', center_color=[0.7,0.7,0.7,1]):
        n_bins = len(bounds) - 1
        ncolors = 2 * n_bins + 1  # impar
        base = cm.get_cmap(cmap_name, ncolors)
        color_arr = base(np.linspace(0,1,ncolors)).copy()
        # El bin central (el que contiene 0) es el de la mitad inferior de n_bins
        zero_bin = np.searchsorted(bounds, 0) - 1
        color_arr[zero_bin + n_bins - (n_bins//2)]# = center_color
        cmap_out = ListedColormap(color_arr[n_bins - (n_bins//2) : n_bins + (n_bins//2) + 1])
        norm = BoundaryNorm(bounds, cmap_out.N)
        return cmap_out, norm

    print("[INFO] Loading data from SQLite file(s)")
    df1 = load_data(sqlite_files[0])

    if len(sqlite_files) == 2:
        df2 = load_data(sqlite_files[1])
        df = pd.merge(
            df2, df1,
            on=['id_stn', 'varno', 'vcoord_bin', 'lat_bin'],
            suffixes=('_exp', '_ctl')
        )
        # DIFERENCIAS SIMPLES
        df['omp'] = df['omp_ctl'] - df['omp_exp']
        df['sigma'] = df['sigma_ctl'] - df['sigma_exp']
        df['n_obs'] = df['n_obs_ctl'] - df['n_obs_exp']

        # --------- ESCALAS SIMÉTRICAS ---------
        # OMP (puedes ajustar los positivos según tus datos)
        bounds_omp_pos = np.array([0.05, 0.1, 0.2, 0.5, 1, 2, 5])
        bounds_omp = np.concatenate((-bounds_omp_pos[::-1], bounds_omp_pos))  # simétrica

        # SIGMA (sólo ejemplo)
        bounds_sigma_pos = np.array([0.05, 0.1, 0.2, 0.5, 1])
        bounds_sigma = np.concatenate((-bounds_sigma_pos[::-1], bounds_sigma_pos))

        # n_obs (de tu ejemplo)
        bounds_nobs_pos = np.array([1, 5, 10, 20, 50, 100, 200])
        bounds_nobs = np.concatenate((-bounds_nobs_pos[::-1], bounds_nobs_pos))

        cmap_omp, norm_omp = custom_div_cbar(bounds_omp, cmap_name='seismic', center_color=[0.7,0.7,0.7,1])
        cmap_sigma, norm_sigma = custom_div_cbar(bounds_sigma, cmap_name='PuOr', center_color=[0.7,0.7,0.7,1])
        cmap_nobs, norm_nobs = custom_div_cbar(bounds_nobs, cmap_name='coolwarm', center_color=[0.7,0.7,0.7,1])

    else:
        df = df1.copy()


  # --------- ESCALAS SIMÉTRICAS ---------
        # OMP (puedes ajustar los positivos según tus datos)
        bounds_omp_pos = np.array([0.05, 0.1, 0.2, 0.5, 1, 2, 5])
        bounds_omp = np.concatenate((-bounds_omp_pos[::-1], bounds_omp_pos))  # simétrica

        # SIGMA (sólo ejemplo)
        bounds_sigma = np.array([0., 0.1, 0.2, 0.5, 1,2,4,5,6])

        # n_obs (de tu ejemplo)
        bounds_nobs_pos = np.array([1, 5, 10, 20, 50, 100, 200])
        bounds_nobs = np.concatenate((-bounds_nobs_pos[::-1], bounds_nobs_pos))
        bounds_nobs = [1, 50, 100, 500, 1000, 2000, 4000, 10000, 24000, 100000]
        cmap_omp, norm_omp = custom_div_cbar(bounds_omp, cmap_name='seismic', center_color=[0.7,0.7,0.7,1])
        cmap_sigma, norm_sigma = custom_div_cbar(bounds_sigma, cmap_name='PuOr', center_color=[0.7,0.7,0.7,1])
        cmap_nobs, norm_nobs = custom_div_cbar(bounds_nobs, cmap_name='jet', center_color=[0.7,0.7,0.7,1])
        import matplotlib.cm as cm

        Ninterv=10
        cmap_nobs = cm.get_cmap('jet', lut=Ninterv)
        norm_nobs = cm.colors.Normalize(vmin=1, vmax=100000)
        y1=[1,50,100,500,1500,2000,4000,10000,16000,24000,100000]

       



    if df.empty:
        print("[WARNING] No data retrieved from database.")
        return

    lat   = df['lat_bin'].values
    vcrd  = df['vcoord_bin'].values
    if family=='sw':
       vcrd  = df['vcoord_bin'].values/100.

    n_obs = df['n_obs'].values
    omp   = df['omp'].values
    sigma = df['sigma'].values

    Delt_LAT = 2
    DeltP = 0.5
    if family=='sw':
       Delt_LAT = 2
       DeltP = 20

    vcoord_min_plot = vcrd.min()
    vcoord_max_plot = vcrd.max()
    lat_min, lat_max = -90, 90

    configs = []
    if len(sqlite_files) == 2:
        configs = [
            {
                'Var': sigma,
                'Nomvar': 'Standard Deviation (DIFF abs(sigma_clt)-abs(sigma_exp) )',
                'cmap': cmap_sigma,
                'norm': norm_sigma,
                'bounds': bounds_sigma,
                'file': os.path.join(pathwork+'/'+family, f'scatterplot_{namesin[0]}_{namesin[1]}_sigma_var{varno}_{id_stn}.png')
            },
            {
                'Var': n_obs,
                'Nomvar': 'Number Of Observations (DIFF n_ctl-n_exp)',
                'cmap': cmap_nobs,
                'norm': norm_nobs,
                'bounds': bounds_nobs,
                'file': os.path.join(pathwork+'/'+family, f'scatterplot_{namesin[0]}_{namesin[1]}_nobs_var{varno}_{id_stn}.png')
            },
            {
                'Var': omp,
                'Nomvar': 'O - P (Bias) (DIFF abs(Bias_clt)-abs(Bias_exp) )',
                'cmap': cmap_omp,
                'norm': norm_omp,
                'bounds': bounds_omp,
                'file': os.path.join(pathwork+'/'+family, f'scatterplot_{namesin[0]}_{namesin[1]}_omp_var{varno}_{id_stn}.png')
            },
        ]
    else:
        configs = [
            {
                'Var': sigma,
                'Nomvar': 'Standard Deviation',
                'cmap': cmap_sigma,
                'norm': norm_sigma,
                'bounds': None,
                'y1':y1,

                'file': os.path.join(pathwork+'/'+family, f'scatterplot_{namesin[0]}_sigma_var{varno}_{id_stn}.png')
            },
            {
                'Var': n_obs,
                'Nomvar': 'Number Of Observations',
                'cmap': cmap_nobs,
                'norm': norm_nobs,
                'bounds': None,
                'y1':y1,
                'file': os.path.join(pathwork+'/'+family, f'scatterplot_{namesin[0]}_nobs_var{varno}_{id_stn}.png')
            },
            {
                'Var': omp,
                'Nomvar': 'O - P (Bias)',
                'cmap': cmap_omp,
                'norm': norm_omp,
                'bounds': None,
                'y1':y1,
                'file': os.path.join(pathwork+'/'+family, f'scatterplot_{namesin[0]}_omp_var{varno}_{id_stn}.png')
            },
        ]

    for conf in configs:
        Var    = conf['Var']
        Nomvar = conf['Nomvar']
        cmap   = conf['cmap']
        norm   = conf['norm']
        bounds = conf.get('bounds', None)
        output_file =conf['file']
        y1=conf['y1']


        fig, ax = plt.subplots(figsize=(14, 7))

        ax.set_xlim(lat_min, lat_max)
        if vcoord_type.lower() == 'Channel':
            ax.set_ylim(vcoord_max_plot, vcoord_min_plot)
        else:
            vcoord_min, vcoord_max = min(vcrd) - DeltP / 2.0, max(vcrd) + DeltP / 2.0
            ax.set_ylim(vcoord_min, vcoord_max)
            if family=='sw':
                 ax.set_ylim(1000, 0)


        # Ticks, colores y mapeo
        if bounds is not None:
            y1 = bounds
        else:
            y1 = np.linspace(np.nanmin(Var), np.nanmax(Var), 12)
        mappable = cm.ScalarMappable(norm=norm, cmap=cmap)
        Colors = [mappable.to_rgba(x) for x in y1]
        inds = np.digitize(Var, y1)

        rects = []
        for i in range(len(n_obs)):
            if not np.isnan(Var[i]):
                val = Var[i]
                if bounds is not None and (val < bounds[0] or val > bounds[-1]):
                    FC = '#FF00FF'
                else:
                    idx = max(0, min(inds[i] - 1, len(Colors) - 1))
                    FC = Colors[idx]
                top = vcrd[i] - DeltP / 2.0
                bottom =  vcrd[i] + DeltP / 2.0
                height = bottom - top
                if height > 0:
                    rect = Rectangle(
                        (lat[i] - Delt_LAT / 2.0, top),
                        Delt_LAT, height, facecolor=FC, edgecolor=FC
                    )
                    rects.append(rect)

        col = PatchCollection(rects, match_original=True)
        ax.add_collection(col)

        ax.set_xlabel('Latitude')
        ax.set_ylabel(f'{vcoord_type}')
        ax.set_title(f'{Nomvar} vs Latitude and {vcoord_type}\nvarno={varno}, id_stn={id_stn}')
        ax.grid(True, color='k')

        left, bottom, width, length = 0.91, 0.15, 0.02, 0.70
        ax_cb = fig.add_axes([left, bottom, width, length])
        if bounds is not None:
            print ("=====================")
            cb2 = colorbar.ColorbarBase(ax_cb, cmap=cmap, norm=norm,
                                        boundaries=bounds,
                                        orientation='vertical', extend='max', ticks=y1)
        else: 
            print ("1111=====================")
            cb2 = colorbar.ColorbarBase(ax_cb, cmap=cmap, norm=norm,
                                        orientation='vertical', extend='max',
                                        spacing='proportional',  ticks=y1)
        cb2.set_label(Nomvar)

        plt.savefig(output_file, format='png')
        plt.close(fig)
        print(f"[INFO] Plot saved to {output_file}")

def zone_plot(
    mode,
    region,
    family,
    id_stn,
    datestart,
    dateend,
    Points,
    boxsizex,
    boxsizey,
    proj,
    pathwork,
    flag_criteria,
    fonction,
    vcoord,
    filesin,
    namesin,
    varno,
    intervales
):
    import pandas as pd
    import matplotlib.pyplot as plt
    from matplotlib.patches import Rectangle
    from matplotlib.collections import PatchCollection
    from matplotlib import cm, colors, colorbar
    import numpy as np
    import sqlalchemy
    import os
    from matplotlib.colors import ListedColormap, BoundaryNorm
    method = 1
    vcoord_type = 'Channel'
    if family == 'sw':
        vcoord_type = 'Pressure(Hpa)'

    debut = datestart
    final = dateend
    sqlite_files = filesin
    os.makedirs(pathwork, exist_ok=True)
    os.makedirs(os.path.join(pathwork, family), exist_ok=True)

    def load_data(sqlite_file):
        engine = sqlalchemy.create_engine(f"sqlite:///{sqlite_file}")
        query = f"""
        SELECT
            id_stn, varno,
            vcoord AS vcoord_bin,
            round(lat/1.0)*1.0 AS lat_bin,
            SUM(sumy)*1.0/SUM(N) AS omp,
            SQRT(SUM(sumy2)*1.0/SUM(N) - POWER(SUM(sumy)*1.0/SUM(N), 2)) AS sigma,
            SUM(N) AS n_obs
        FROM moyenne
        WHERE
            varno = {varno}
            AND id_stn = '{id_stn}'
            AND sumy IS NOT NULL
            AND date BETWEEN '{debut}' AND '{final}'
        GROUP BY id_stn, varno, vcoord_bin, lat_bin
        HAVING COUNT(*) > 1
        ORDER BY vcoord_bin DESC, lat_bin
        """
        return pd.read_sql_query(query, engine)

    def custom_div_cbar(bounds, cmap_name='seismic', center_color=[0.7, 0.7, 0.7, 1]):
        n_bins = len(bounds) - 1
        ncolors = 2 * n_bins + 1
        base = cm.get_cmap(cmap_name, ncolors)
        color_arr = base(np.linspace(0, 1, ncolors)).copy()
        zero_bin = np.searchsorted(bounds, 0) - 1
        cmap_out = ListedColormap(color_arr[n_bins - (n_bins // 2): n_bins + (n_bins // 2) + 1])
        norm = BoundaryNorm(bounds, cmap_out.N)
        return cmap_out, norm

    df1 = load_data(sqlite_files[0])

    if len(sqlite_files) == 2:
        df2 = load_data(sqlite_files[1])
        df = pd.merge(df2, df1, on=['id_stn', 'varno', 'vcoord_bin', 'lat_bin'], suffixes=('_exp', '_ctl'))
        df['omp'] = df['omp_ctl'] - df['omp_exp']
        df['sigma'] = df['sigma_ctl'] - df['sigma_exp']
        df['n_obs'] = df['n_obs_ctl'] - df['n_obs_exp']

        bounds_omp_pos = np.array([0.05, 0.1, 0.2, 0.5, 1, 2, 5])
        bounds_omp = np.concatenate((-bounds_omp_pos[::-1], bounds_omp_pos))
        bounds_sigma_pos = np.array([0.05, 0.1, 0.2, 0.5, 1])
        bounds_sigma = np.concatenate((-bounds_sigma_pos[::-1], bounds_sigma_pos))
        bounds_nobs_pos = np.array([1, 50, 100, 500 ])
        bounds_nobs = np.concatenate((-bounds_nobs_pos[::-1], bounds_nobs_pos)) 
        cmap_sigma, norm_sigma = custom_div_cbar(bounds_sigma, 'PuOr')
        cmap_omp, norm_omp = custom_div_cbar(bounds_omp, 'seismic')
        cmap_nobs = cm.get_cmap('jet', len(bounds_nobs) - 1)
        norm_nobs = BoundaryNorm(bounds_nobs, cmap_nobs.N)

    else:
        df = df1.copy()
        bounds_omp_pos = np.array([0.05, 0.1, 0.2, 0.5, 1, 2, 5])
        bounds_omp = np.concatenate((-bounds_omp_pos[::-1], bounds_omp_pos))
        bounds_sigma = np.array([0., 0.1, 0.2, 0.5, 1, 2, 4, 5, 6])
        bounds_nobs = [1, 50, 100, 500, 1000, 2000, 4000, 10000, 24000, 100000]

        cmap_omp, norm_omp = custom_div_cbar(bounds_omp, 'seismic')
        cmap_sigma, norm_sigma = custom_div_cbar(bounds_sigma, 'PuOr')
        cmap_nobs = cm.get_cmap('jet', len(bounds_nobs) - 1)
        norm_nobs = BoundaryNorm(bounds_nobs, cmap_nobs.N)

    if df.empty:
        print("[WARNING] No data retrieved from database.")
        return
    variable_name, units, vcoord_type = pikobs.type_varno(varno)
    print (variable_name, units, vcoord_type)
    lat = df['lat_bin'].values
    vcrd = df['vcoord_bin'].values / 100. if family == 'sw' else df['vcoord_bin'].values
    omp = df['omp'].values
    sigma = df['sigma'].values
    n_obs = df['n_obs'].values

    Delt_LAT, DeltP = (2, 20) if family == 'sw' else (2, 0.5)
    vcoord_min_plot, vcoord_max_plot = vcrd.min(), vcrd.max()
    lat_min, lat_max = -91, 91

    variables = [
        ('sigma', sigma, 'Standard Deviation', cmap_sigma, norm_sigma, bounds_sigma),
        ('nobs', n_obs, 'Number Of Observations', cmap_nobs, norm_nobs, bounds_nobs),
        ('omp', omp, 'O - P (Bias)', cmap_omp, norm_omp, bounds_omp)
    ]


    # Arrays 1D
    lat = np.asarray(lat).ravel()
    vcrd = np.asarray(vcrd).ravel()
    
    # X-lims (latitud) con margen medio Delt_LAT
    lat_min = np.min(lat) - Delt_LAT / 2.0
    lat_max = np.max(lat) + Delt_LAT / 2.0
    
    # Canales únicos y mapeo a índice (para modo 'indices')
    unique_vcrd = np.sort(np.unique(vcrd))
    vcrd_to_idx = {val: idx for idx, val in enumerate(unique_vcrd)}
    vcrd_idx = np.array([vcrd_to_idx[val] for val in vcrd])
    
    # Precalcular celdas (bordes y alturas) en modo 'values' para DeltP_val auto (variable)
    def compute_variable_heights(unique_vals):
        # Si solo hay un valor, asume altura 1.0
        if len(unique_vals) == 1:
            edges = np.array([unique_vals[0] - 0.5, unique_vals[0] + 0.5])
        else:
            centers = unique_vals
            mids = (centers[:-1] + centers[1:]) / 2.0
            # bordes extremos por extrapolación
            first_edge = centers[0] - (centers[1] - centers[0]) / 2.0
            last_edge = centers[-1] + (centers[-1] - centers[-2]) / 2.0
            edges = np.concatenate(([first_edge], mids, [last_edge]))
        heights = edges[1:] - edges[:-1]
        heights_by_val = {val: heights[i] for i, val in enumerate(unique_vals)}
        return edges, heights_by_val
    
    # Elegir modo según 'family'
    if family == 'sw':
        # Modo valores reales, eje 0..1000 e invertido (0 arriba).
        tipo = 'values'
        y_limits = (0, 1000)
        invertir_y = True           # pon False si prefieres 0 abajo, 1000 arriba
        DeltP_val = None            # AUTO: altura variable según discretización real
        y_major_step = 100
        y_minor_step = 10
        # Prepara alturas variables
        edges_vals, heights_by_val = compute_variable_heights(unique_vcrd)
    else:
        # Modo compacto por canales (categorías)
        tipo = 'indices'
        invertir_y = True           # canal menor arriba; pon False si lo quieres abajo
        y_limits = None
        DeltP_val = 1.0             # no se usa en 'indices'
        y_major_step = None
        y_minor_step = None
        edges_vals, heights_by_val = None, None
    
    # Directorio de salida
    outdir = os.path.join(pathwork, family)
    os.makedirs(outdir, exist_ok=True)
    
    # Bucle por variables
    for name, var, label, cmap, norm, bounds in variables:
        var = np.asarray(var).ravel()
        if var.shape[0] != lat.shape[0]:
            raise ValueError(f"var '{name}' tiene longitud {var.shape[0]} y no coincide con lat ({lat.shape[0]}).")
    
        fig, ax = plt.subplots(figsize=(14, 14))
        ax.set_xlim(lat_min, lat_max)
    
        # Mapeo de colores
        mappable = cm.ScalarMappable(norm=norm, cmap=cmap)
        colors_list = [mappable.to_rgba(x) for x in bounds]
        inds = np.digitize(var, bounds)
    
        # Crear rectángulos
        rects = []
        for i in range(len(var)):
            vi = var[i]
            if not np.isnan(vi):
                if vi < bounds[0] or vi > bounds[-1]:
                    FC = '#FF00FF'  # fuera de rango
                else:
                    idx_color = max(0, min(inds[i] - 1, len(colors_list) - 1))
                    FC = colors_list[idx_color]
    
                if tipo == 'indices':
                    y_center = int(vcrd_idx[i])
                    y_height = 1.0
                else:
                    y_center = vcrd[i]
                    # Si DeltP_val es None -> usar altura variable por discretización
                    if DeltP_val is None and heights_by_val is not None:
                        # Puede fallar si vcrd[i] no está exactamente en unique_vcrd por flotantes;
                        # en ese caso buscamos el índice del valor más cercano.
                        key = y_center
                        if key not in heights_by_val:
                            idx_near = int(np.argmin(np.abs(unique_vcrd - y_center)))
                            key = unique_vcrd[idx_near]
                        y_height = float(heights_by_val[key])
                    else:
                        y_height = float(DeltP_val)
    
                rects.append(Rectangle(
                    (lat[i] - Delt_LAT / 2.0, y_center - y_height / 2.0),
                    Delt_LAT, y_height,
                    facecolor=FC, edgecolor=FC
                ))
    
        ax.add_collection(PatchCollection(rects, match_original=True))
    
        # Configuración del eje Y y rejilla
        if tipo == 'indices':
            N = len(unique_vcrd)
            pad = 0.5  # margen para que no se “coman” las celdas extremas
            ymin, ymax = -0.5 - pad, N - 0.5 + pad
            ax.set_ylim((ymax, ymin) if invertir_y else (ymin, ymax))
    
            # Ticks mayores en centros (etiquetas reales), menores en bordes (rejilla)
            ax.set_yticks(np.arange(N))
            #ax.set_yticklabels(int(unique_vcrd), fontsize=6)
            ax.set_yticklabels([f"{v:.0f}" for v in unique_vcrd], fontsize=6)
            ax.set_yticks(np.arange(-0.5, N + 0.5, 1.0), minor=True)
    
            ax.grid(False)
            ax.grid(True, axis='y', which='minor', color='k', linewidth=0.8)
            ax.grid(True, axis='x', which='major', color='k', alpha=0.3)
            ax.set_ylabel(vcoord_type)
        else:
            # Modo valores reales (0..1000 si family == 'ee')
            if y_limits is not None:
                ymin, ymax = y_limits
            else:
                # Si tenemos bordes calculados, usarlos para no cortar
                if edges_vals is not None:
                    ymin, ymax = float(edges_vals[0]), float(edges_vals[-1])
                else:
                    ymin = float(np.nanmin(vcrd) - 0.5 * (1.0 if DeltP_val is None else DeltP_val))
                    ymax = float(np.nanmax(vcrd) + 0.5 * (1.0 if DeltP_val is None else DeltP_val))
    
            ax.set_ylim((ymax, ymin) if invertir_y else (ymin, ymax))
    
            # Ticks y rejileta
            if y_major_step is None or y_minor_step is None:
                span = abs(ymax - ymin)
                if y_major_step is None:
                    y_major_step = 100 if span >= 800 else 50 if span >= 300 else 10
                if y_minor_step is None:
                    y_minor_step = max(y_major_step / 5.0, 1)
            import matplotlib.ticker as mticker   
           # ax.yaxis.set_major_locator(MultipleLocator(y_major_step))
           # ax.yaxis.set_minor_locator(MultipleLocator(y_minor_step))
            ax.yaxis.set_major_locator(mticker.MultipleLocator(y_major_step))
            ax.yaxis.set_minor_locator(mticker.MultipleLocator(y_minor_step))
            ax.grid(True, axis='y', which='major', color='k', alpha=0.4)
            ax.grid(True, axis='x', which='major', color='k', alpha=0.3)
            ax.set_ylabel(vcoord_type)
    
        ax.set_xlabel('Latitud')
        ax.set_title(f'{label} vs Latitude and Channel from {datestart} to {dateend},\nvarno={varno}, id_stn={id_stn}')
    
        # Barra de color
        cax = fig.add_axes([0.91, 0.15, 0.02, 0.7])
        cb = colorbar.ColorbarBase(
            cax, cmap=cmap, norm=norm, boundaries=bounds,
            orientation='vertical', ticks=bounds
        )
        cb.set_label(label)
    
        # Guardado alta resolución
        output_file = os.path.join(outdir, f'1scatterplot_{"_".join(namesin)}_{name}_var{varno}_{id_stn}.png')
        plt.savefig(output_file, dpi=600, format='png', bbox_inches='tight', pad_inches=0.05)
        plt.close(fig)
#    for name, var, label, cmap, norm, bounds in variables:
#        fig, ax = plt.subplots(figsize=(14, 7))
#        ax.set_xlim(lat_min, lat_max)
#        if vcoord_type.lower() == 'channel':
#            ax.set_ylim(vcoord_max_plot, vcoord_min_plot)
#        else:
#            ax.set_ylim(1000, 0) if family == 'sw' else ax.set_ylim(vcoord_min_plot, vcoord_max_plot)
#
#        mappable = cm.ScalarMappable(norm=norm, cmap=cmap)
#        colors_list = [mappable.to_rgba(x) for x in bounds]
#        inds = np.digitize(var, bounds)
#
#        rects = []
#        for i in range(len(var)):
#            if not np.isnan(var[i]):
#                val = var[i]
#                if val < bounds[0] or val > bounds[-1]:
#                    FC = '#FF00FF'
#                else:
#                    idx = max(0, min(inds[i] - 1, len(colors_list) - 1))
#                    FC = colors_list[idx]
#                rect = Rectangle(
#                    (lat[i] - Delt_LAT / 2.0, vcrd[i] - DeltP / 2.0),
#                    Delt_LAT, DeltP,
#                    facecolor=FC, edgecolor=FC
#                )
#                rects.append(rect)
#
#        ax.add_collection(PatchCollection(rects, match_original=True))
#        ax.set_xlabel('Latitude')
#        ax.set_ylabel(vcoord_type)
#        ax.set_title(f'{label} vs Latitude and {vcoord_type} from {datestart} to {dateend},\nvarno={varno}, id_stn={id_stn}')
#        ax.grid(True, color='k')
#
#        cax = fig.add_axes([0.91, 0.15, 0.02, 0.7])
#        cb = colorbar.ColorbarBase(cax, cmap=cmap, norm=norm, boundaries=bounds,
#                                   orientation='vertical', ticks=bounds)
#        cb.set_label(label)
#
#        output_file = os.path.join(pathwork, family, f'scatterplot_{"_".join(namesin)}_{name}_var{varno}_{id_stn}.png')
#        plt.savefig(output_file, format='png')
#        plt.close(fig)
#



"""
Generating Cardiograms for Radiance Assimilation Analysis
=========================================================

This script is designed to generate cardiograms that analyze the results of radiance assimilation. Below is a description of the generated graphs, their characteristics, and their importance in the context of data assimilation.

Cardiograms of Radiance Assimilation
-------------------------------------
The script generates four main types of cardiograms, each providing critical information to assess the performance of radiance assimilation.

.. image::  ../../../docs/source/_static/cardio.png
   :alt: code_quality

1. **Mean of Differences**
   This cardiogram shows the averages of the differences between observations and predictions (O-P) and, if available, the differences between observations and adjustments (O-A). It also includes bias corrections (Bcor) if present.
   **Importance:** Analyzing the averages of O-P and O-A helps identify and quantify any systematic bias in the assimilation process, which is crucial for improving the accuracy of prediction models.

2. **Standard Deviations of Differences**
   This cardiogram shows the standard deviations of the differences between observations and predictions (O-P) and, if available, the standard deviations of the differences between observations and adjustments (O-A).
   **Importance:** Standard deviations measure the dispersion of errors. Analyzing these deviations helps understand the variability and uncertainty in the predictions and adjustments made during radiance assimilation.

3. **Obsvalue of Data**
   This cardiogram shows the obsvalue in the assimilation process.
   **Importance:** Evaluating the obsvalue of accepted data is essential to understand the efficiency of the assimilation system and the quality of the data being used. A high number of accepted data points can indicate an efficient and reliable filtering process.

4. **Total Number of Processed Data (Data Processing Analysis)**
   This cardiogram shows the total number of processed data points (Ntot) during assimilation.
   **Importance:** The total number of processed data points provides an overview of the amount of information handled by the system. It is an indicator of the data volume and the system's capacity to process large datasets.


*******************************************************
Generate Cardiograms for Radiance Assimilation Analysis
*******************************************************

To start an interactive session for generating cardiograms, use the following qsub command:
::
    qsub -I -X -l select=4:ncpus=80:mpiprocs=80:ompthreads=1:mem=185gb -l place=scatter -l walltime=6:0:0

Generating Cardiograms
^^^^^^^^^^^^^^^^^^^^^^
To generate cardiograms using pikobs, use the following command format:
::

    python -c 'import pikobs; pikobs.cardio.arg_call()'\

         --path_experience  /home/sprj700/data_maestro/ppp6/maestro_archives/G2FC900V2E22/monitoring/banco/postalt/ \

         --experience_name     G2FC900V2E22               \

         --pathwork   to_amsua_qc          \

         --datestart  2022061400                \

         --dateend    2022062400                \

         --region     Monde                     \

         --family     ai            \

         --flags_criteria assimilee             \

         --id_stn     join     \

         --channel    all        \

         --plot_title Test                      \

         --n_cpu      40

Parameters Explanation
^^^^^^^^^^^^^^^^^^^^^^
- **path_experience:** Path to the directory where the experience data is stored.
- **experience_name:** Name of the specific experience.
- **pathwork:** Directory path indicating the working directory for the script.
- **datestart:** Start date and time of the assimilation process (format: YYYYMMDDHH).
- **dateend:** End date and time of the assimilation process (format: YYYYMMDDHH).
- **region:** Geographic region or area of interest for the assimilation analysis. (e.g., Monde, PoleNord, PoleSud, AmeriqueduNord, OuestAmeriqueduNord, AmeriqueDuNordPlus, ExtratropiquesNord, HemisphereNord, HemisphereSud, Asie, Europe, Mexique, Canada, BaieDhudson, Arctiquecanadien, EtatsUnis, Tropiques30, Tropiques, Australie, Pacifique and Atlantique)
- **family:** Family of observation (e.g., mwhs2, mwhs2_qc, to_amsua_qc, to_amsua, to_amsua_allsky, to_amsua_allsky_qc, to_amsub_qc, to_amsub, ssmis_qc, ssmis, iasi, iasi_qc, crisfsr1_qc, crisfsr2_qc, cris, atms_allsky, atms_qc and csr, csr_qc)
- **flags_criteria:** Specific flag criteria used during assimilation (e.g., all, assimilee, bgckalt, bgckalt_qc and postalt).
- **id_stn:** all or satellites IDs for which assimilation results are analyzed also join satellites (e.g., METOP-1, METOP-3). Modify as needed to specify particular satellites.
- **channel:** all or specific channels to analyze. Use 'all' to analyze all available channels. Modify as needed to specify particular channels of interest.
- **plot_title:** Title for the generated cardiogram.
- **n_cpu:** Number of CPU cores to be used for parallel processing. Adjust according to your system capabilities.

Additional Note
^^^^^^^^^^^^^^^
- Modify **--channel** and **--id_stn** parameters as needed to specify particular satellites and channels of interest.
- Ensure paths (**path_experience, pathwork**) are correctly set to point to your data directories.
- The script leverages parallel processing (**--n_cpu**) to optimize performance, adjust according to your system capabilities.
- The generated cardiograms provide comprehensive visual insights into the assimilation performance metrics across selected satellites, channels, and criteria.

""" 
#!/usr/bin/python3
import sqlite3
import pikobs
import re
import os
from  dask.distributed import Client
import numpy as np
import sqlite3
import os
import re
import sqlite3
from  datetime import datetime, timedelta


def create_serie_cardio(family, 
                        new_db_filename, 
                        existing_db_filename,
                        region_seleccionada,
                        selected_flags, 
                        id_stn,
                        channel, 
                        varnos):
    """

    Create a new SQLite database with a 'moyenne' table and populate it with data from an existing database.

    Args:

      new_db_filename (str): Filename of the new database to be created.
  
      existing_db_filename (str): Filename of the existing database to be attached.
  
      region_seleccionada (str): Region selection criteria.
   
      selected_flags (str): Selected flags criteria.
   
      FONCTION (float): Value for sum_fonction column.

    
    Returns:
    
      None
    
    """
 
    pattern = r'(\d{10})'
    match = re.search(pattern, existing_db_filename)

    if match:
        date = match.group(1)
       
    else:
        print("No 10 digits found in the string.")
    
    # Connect to the new database
    if id_stn[0]=='all':
      criteria_id_stn = '    '
    else:
       elemnts = [f"'{element}'" for element in id_stn]

       resultado = f"({','.join(elemnts)})"
       criteria_id_stn = f'   '
    if channel[0]=='all':
      criteria_vcoord = '    '
    else:
       resultado = f"({','.join(channel)})"
       criteria_vcoord = f'    '

    
    filename=f"file::memory:?cache=shared"
    new_db_conn = sqlite3.connect(filename, uri=True, isolation_level=None, timeout=999999)
    new_db_cursor = new_db_conn.cursor()

    FAM, VCOORD, VCOCRIT, STATB, element, VCOTYP = pikobs.family(family)
    if id_stn[0]!='all' and id_stn[0]!='join':
       id_stn[0]='all' 
    if varnos:
        element=",".join(varnos)
    if channel=='join':
        VCOORD='  vcoord '
    if channel[0]=='join' and  id_stn[0]=='all':
      group_channel = ' "join" as Chan,    '
      group_id_stn  = ' id_stn as id_stn '
      group_id_stn_vcoord = ' group by id_stn,varno'
    if channel[0]=='all' and  id_stn[0]=='join':
      group_channel = f' {VCOORD} as  Chan, '
      group_id_stn  = ' "join" as id_stn '
      group_id_stn_vcoord = f' group by {VCOORD},varno'
    if channel[0]=='all' and  id_stn[0]=='all':
      group_channel = f'  {VCOORD} as Chan,'
      group_id_stn  = ' id_stn as id_stn '
      group_id_stn_vcoord = f' group by id_stn, {VCOORD},varno'

    if channel[0]=='join' and  id_stn[0]=='join':
      group_channel = ' "join" as Chan, '
      group_id_stn  =  ' "join" as id_stn  '
      group_id_stn_vcoord = 'group by varno  '

    LAT1, LAT2, LON1, LON2 = pikobs.regions(region_seleccionada)
    LATLONCRIT = pikobs.generate_latlon_criteria(LAT1, LAT2, LON1, LON2)
    flag_criteria = pikobs.flag_criteria(selected_flags)

    # Attach the existing database
    new_db_cursor.execute(f"ATTACH DATABASE '{existing_db_filename}' AS db;")
    # load extension CMC 
    new_db_conn.enable_load_extension(True)
    extension_dir = f'{os.path.dirname(pikobs.__file__)}/extension/libudfsqlite-shared.so'
    new_db_conn.execute(f"SELECT load_extension('{extension_dir}')")
    # Create the 'moyenne' table in the new database if it doesn't exist
    new_db_cursor.execute("""
           CREATE TABLE IF NOT EXISTS serie_cardio ( 
            DATE INTEGER,
            Chan INTEGER,
            Nrej INTEGER,
            Nacc INTIGER,
            AvgOMP FLOAT, 
            AvgOMA FLOAT,
            StdOMP FLOAT,
            StdOMA FLOAT,
            NDATA  INTEGER,
            Nprofile INTEGER,
            AvgBCOR FLOAT,
            AvgOBS FLOAT,
            Ntot INTEGER,
            varno INTEGER,
            id_stn  TEXT
        );
    """)
    query=f"""INSERT INTO serie_cardio (

            DATE,
            Chan,
            Nrej,
            Nacc,
            AvgOMP, 
            AvgOMA,
            StdOMP,
            StdOMA,
            NDATA,
            Nprofile,
            AvgBCOR,
            AvgOBS,
            Ntot,
            varno,
            id_stn
        )
    
    
             SELECT 
                 isodatetime({date}) AS DATE,  
                 {group_channel}
                 SUM(flag & 512=512) AS Nrej,
                 SUM(flag & 4096=4096) AS Nacc,
                 ROUND(AVG(OMP), 4) AS AvgOMP,
                 ROUND(AVG(OMA), 4) AS AvgOMA,
                 ROUND(STDDEV(OMP), 4) AS StdOMP,
                 ROUND(STDDEV(OMA), 4) AS StdOMA,
                 SUM(OMP IS NOT NULL) AS NDATA,
                 COUNT(DISTINCT id_obs) AS Nprofils,
                 ROUND(AVG(BIAS_CORR), 4) AS AvgBCOR,
                 ROUND(AVG(OBSVALUE), 4) AS AvgOBS,
                 (SELECT COUNT(*) FROM header h2 WHERE h2.ID_STN = header.ID_STN) AS Ntot,
                 varno AS varno,
                 {group_id_stn}
                 
            FROM 
                 header
             NATURAL JOIN 
                 data
             WHERE 
                 varno IN ({element}) 
                 {criteria_id_stn}
                 {criteria_vcoord}
                 {flag_criteria}
                 {LATLONCRIT}
                 {VCOCRIT}
                {group_id_stn_vcoord};"""
    new_db_cursor.execute(query)
    # Commit changes and detach the existing database
    new_db_conn.commit()

    # Commit changes and detach the existing database
    new_db_cursor.execute("DETACH DATABASE db;")
    try:
         combine(new_db_filename, filename)
    except sqlite3.Error as error:
         print(f"Error while creating a single sqlite file:  {os.path.basename(filename)}", error)

    
    # Close the connections
    #anew_db_conn.close()


def combine(pathfileout, filememory):

    """ Creating a single sqlite file from multiple sqlite files
 
    args:
    ----------------

     pathfileout   : output averaging  sqlite file 
     filememory    : name of the sqlite file in memory to copy
   
    output:
    ----------------

    Nothing
     A sqlite file is made with all averaged volume scans
   
   """

    # write in output averaging  sqlite file
    conn_pathfileout = sqlite3.connect(pathfileout, uri=True, isolation_level=None, timeout=9999)

    conn_pathfileout.execute("""PRAGMA journal_mode=OFF;""")

    # off the journal
    create_table_if_not_exists( conn_pathfileout)
    conn_pathfileout.execute("""PRAGMA journal_mode=OFF;""")
    # SQLite continues without syncing as soon as it has handed data off to the operating system
    conn_pathfileout.execute("""PRAGMA synchronous=OFF;""")
    # Wait to read and write until the next process finishes
    # attach the sqlite file in memory for one PPI
    create_table_if_not_exists(conn_pathfileout)

    conn_pathfileout.execute("ATTACH DATABASE ? AS this_avg_db;", (filememory,))
    order_sql = """ INSERT INTO serie_cardio( DATE, Chan, Nrej, Nacc, AvgOMP, AvgOMA, StdOMP, StdOMA, NDATA, Nprofile, AvgBCOR, AvgOBS, Ntot, varno, id_stn
 

                )
                    SELECT
                    DATE, Chan, Nrej, Nacc, AvgOMP, AvgOMA, StdOMP, StdOMA, NDATA, Nprofile, AvgBCOR, AvgOBS, Ntot, varno, id_stn
 
                    FROM  this_avg_db.serie_cardio"""
    conn_pathfileout.execute(order_sql) 

    conn_pathfileout.commit()
    conn_pathfileout.execute(""" DETACH DATABASE this_avg_db """)

def create_table_if_not_exists(cursor):
    """
    Create the 'serie_cardio' table if it does not already exist.

    Args:
        cursor (sqlite3.Cursor): Database cursor for executing SQL queries.
    """
    query = """
           CREATE TABLE IF NOT EXISTS serie_cardio ( 
            DATE INTEGER,
            Chan INTEGER,
            Nrej INTEGER,
            Nacc INTIGER,
            AvgOMP FLOAT, 
            AvgOMA FLOAT,
            StdOMP FLOAT,
            StdOMA FLOAT,
            NDATA  INTEGER,
            Nprofile INTEGER,
            AvgBCOR FLOAT,
            AvgOBS FLOAT,
            Ntot INTEGER,
            varno INTEGER,
            id_stn  TEXT );
    """
    cursor.execute(query)


def create_data_list_cardio(datestart1, 
                            dateend1,
                            families,
                            pathin, 
                            pathwork,
                            flag_criteria,
                            id_stn,
                            channel,
                            regions,
                            varnos):
    
    data_list_cardio = []
    for family in families:
        for region in regions:
             # Convert datestart and dateend to datetime objects
             datestart = datetime.strptime(datestart1, '%Y%m%d%H')
             dateend = datetime.strptime(dateend1, '%Y%m%d%H')

             # Initialize the current_date to datestart
             current_date = datestart

             # Define a timedelta of 6 hours
             delta = timedelta(hours=6)
             FAM, VCOORD, VCOCRIT, STATB, element, VCOTYP = pikobs.family(family)

            #  print ("VCOORD", vcoord, element, type(element))
              # Iterate through the date range in 6-hour intervals
             while current_date <= dateend:
                 # Format the current date as a string
                 formatted_date = current_date.strftime('%Y%m%d%H')

                 # Build the file name using the date and family
                 filename = f'{formatted_date}_{family}'

                 file_path_name = f'{pathin}/{filename}'
                 conn = sqlite3.connect(file_path_name)
                 # Create a cursor to execute SQL queries
                 cursor = conn.cursor()

                 #  Create a new dictionary and append it to the list
                 data_dict = {'family': family,
                                   'filein': f'{pathin}/{filename}',
                                   'db_new': f'{pathwork}/{family}/cardio_{region}_{datestart1}_{dateend1}_{flag_criteria}_{family}.db',
                                   'region': region,
                                   'flag_criteria': flag_criteria,
                                   'varno':  varnos,
                                   'vcoord': channel,
                                   'id_stn': id_stn}
                 data_list_cardio.append(data_dict)
                 conn.close()

                 # Update the current_date in the loop by adding 6 hours
                 current_date += delta

    return data_list_cardio


def get_station_ids(cursor, id_stn):
    if id_stn[0] == 'all':
        cursor.execute("SELECT DISTINCT id_stn FROM serie_cardio;")
        return [item[0] for item in cursor.fetchall()]
    return np.array(id_stn)

def fetch_channels_and_varno(cursor, idstn, channel):
    criter = f'WHERE id_stn = "{idstn}"'
    if channel[0] == 'all':
        if idstn=='join':
           query = f"SELECT DISTINCT chan, varno FROM serie_cardio  ;"
        else:
           query = f"SELECT DISTINCT chan, varno FROM serie_cardio {criter} ;"       
        cursor.execute(query)
        return cursor.fetchall()
    else:
        channels_varno = []
        for chan in np.array(channel):
            if idstn=='join':
               query = f"SELECT DISTINCT 'join', varno FROM serie_cardio;"
            else:
               query = f"SELECT DISTINCT 'join', varno FROM serie_cardio;"        
            cursor.execute(query)
            result = cursor.fetchone()
            if result:
                channels_varno.append(result)
        return channels_varno

def create_data_list_plot(datestart1, dateend1, families, pathin, pathwork, flag_criteria, regions, id_stn, channel):
    data_list_plot = []
    for family in families:
        for region in regions: 
           filedb = f'{pathwork}/{family}/cardio_{region}_{datestart1}_{dateend1}_{flag_criteria}_{family}.db'
           try:
               with sqlite3.connect(filedb) as conn:
                   cursor = conn.cursor()
                   if id_stn[0]=='all':
                       id_stns = get_station_ids(cursor, id_stn)
                   elif id_stn[0]=='join':  
                       id_stns = ['join']
                   else:
                       id_stns=id_stn
                   for idstn in id_stns:
                       channels_varno = fetch_channels_and_varno(cursor, idstn, channel)
                       for chan, varno in channels_varno:
                           data_dict_plot = {
                               'id_stn': idstn,
                               'vcoord': chan,
                               'varno': varno,
                               'region':region,
                               'family':family,
                           }
                           data_list_plot.append(data_dict_plot)
           except sqlite3.Error as e:
               print(f"Database error: {e}")
           except Exception as e:
               print(f"Error: {e}")
    return data_list_plot

def make_cardio(path_experience_files,
                experience_names,      
                pathwork, 
                datestart,
                dateend,
                regions, 
                families, 
                flag_criteria, 
                id_stn,
                channel,
                plot_type,
                plot_title,
                n_cpu,
                varnos):
       
       for family in families:
           pikobs.delete_create_folder(pathwork, family) 

       data_list_cardio = create_data_list_cardio(datestart,
                                           dateend, 
                                           families, 
                                           path_experience_files, 
                                           pathwork,
                                           flag_criteria, 
                                           id_stn,
                                           channel,
                                           regions,
                                           varnos)

       import time
       import dask
       t0 = time.time()
       if n_cpu==1:
        print (f'in Serie: {len(data_list_cardio)} files for serie calculation')
        for  data_ in data_list_cardio:  
            create_serie_cardio(data_['family'], 
                                data_['db_new'], 
                                data_['filein'],
                                data_['region'],
                                data_['flag_criteria'],
                                data_['id_stn'],
                                data_['vcoord'],
                                data_['varno'])




       else:
        print (f'in Paralle: {len(data_list_cardio)} files for serie calculation')
        with dask.distributed.Client(processes=True, threads_per_worker=1, 
                                           n_workers=n_cpu, 
                                           silence_logs=40) as client:
            delayed_funcs = [dask.delayed(create_serie_cardio)(data_['family'], 
                                              data_['db_new'], 
                                              data_['filein'],
                                              data_['region'],
                                              data_['flag_criteria'],
                                              data_['id_stn'],
                                              data_['vcoord'],
                                              data_['varno'])for data_ in data_list_cardio]
            results = dask.compute(*delayed_funcs)
        
       tn= time.time()
       print ('Total time serie calculation:', round(tn-t0,2) )  
       data_list_plot = create_data_list_plot(datestart,
                                    dateend, 
                                    families, 
                                    path_experience_files, 
                                    pathwork,
                                    flag_criteria, 
                                    regions,
                                    id_stn,
                                    channel)



       t0 = time.time()
       if len(data_list_plot)==0:
          raise ValueError('You must specify best selectoion od id_stn and channel plot==0')

       if n_cpu==1: 
          print (f'in Serie plots = {len(data_list_plot)}')
          for  data_ in data_list_plot:  
              pikobs.cardio_plot(pathwork,
                                 datestart,
                                 dateend,
                                 flag_criteria,
                                 data_['family'],
                                 plot_title,
                                 plot_type, 
                                 data_['vcoord'],
                                 data_['id_stn'], 
                                 data_['varno'],
                                 data_['region'])
       else:
          print (f'in Paralle = {len(data_list_plot)} plots')
          with dask.distributed.Client(processes=True, threads_per_worker=1, 
                                           n_workers=n_cpu, 
                                           silence_logs=40) as client:
            delayed_funcs = [dask.delayed(pikobs.cardio_plot)(
                               pathwork,
                                 datestart,
                                 dateend,
                                 flag_criteria,
                                 data_['family'],
                                 plot_title,
                                 plot_type, 
                                 data_['vcoord'],
                                 data_['id_stn'], 
                                 data_['varno'],
                                 data_['region'])for data_ in data_list_plot]

            results = dask.compute(*delayed_funcs)
       tn= time.time()
       print ('Total time for plotting:',tn-t0 )  

 



def arg_call():
    import argparse
    import sys

    parser = argparse.ArgumentParser()
    parser.add_argument('--path_experience_files', default='undefined', type=str, help="Directory where input sqlite files are located")
    parser.add_argument('--experience_name', default='undefined', type=str, help="experience's name")
    parser.add_argument('--pathwork', default='undefined', type=str, help="Working directory")
    parser.add_argument('--datestart', default='undefined', type=str, help="Start date")
    parser.add_argument('--dateend', default='undefined', type=str, help="End date")
    parser.add_argument('--region', nargs="+", default='undefined', type=str, help="Region")
    parser.add_argument('--family', nargs="+", default='undefined', type=str, help="Family")
    parser.add_argument('--flags_criteria', default='undefined', type=str, help="Flags criteria")
    parser.add_argument('--id_stn',nargs="+", default='all', type=str, help="id_stn") 
    parser.add_argument('--channel',nargs="+",  default='all', type=str, help="channel") 
    parser.add_argument('--plot_type', default='wide', type=str, help="channel")
    parser.add_argument('--plot_title', default='plot', type=str, help="channel")
    parser.add_argument('--n_cpus', default=1, type=int, help="Number of cpus")
    parser.add_argument('--varnos', nargs="+", default='undefined', type=str, help="Function")
   
    args = parser.parse_args()
    print ( "Inputs in cardiogram calculation")
    print ("----------------------------------------")

    for arg in vars(args):
      
       print (f'--{arg} {getattr(args, arg)}')
    print ("----------------------------------------")
    # check if each argument is 'undefined'
    if args.path_experience_files == 'undefined':
        raise ValueError('You must specify --path_experience_files')
    if args.experience_name == 'undefined':      
        raise ValueError('You must specify -experience_name') 
    if args.varnos == 'undefined':
        args.varnos = []
    if args.pathwork  == 'undefined':
        raise ValueError('You must specify --pathwork')
    if args.datestart == 'undefined':
        raise ValueError('You must specify --datestart')
    if args.dateend == 'undefined':
        raise ValueError('You must specify --dateend')
    if args.region == 'undefined':
        raise ValueError('You must specify --region')
    if args.family == 'undefined':
        raise ValueError('You must specify --family')
    if args.flags_criteria == 'undefined':
        raise ValueError('You must specify --flags_criteria')
   # if args.fonction == 'undefined':
   #     raise ValueError('You must specify --fonction')

    #Call your function with the arguments
    sys.exit(make_cardio (args.path_experience_files,
                          args.experience_name,
                          args.pathwork,
                          args.datestart,
                          args.dateend,
                          args.region,
                          args.family,
                          args.flags_criteria,
                          args.id_stn,
                          args.channel,
                          args.plot_type,
                          args.plot_title,
                          args.n_cpus,
                          args.varnos))




def type_boxes(fonction):
   if fonction=='omp':
       FNAM  = 'O-P'
       FNAMP = 'O-P'
       SUM   = 'SUMX'
       SUM2  = 'SUMX2'
   if fonction=='stdomp':
       FNAM  = 'O-P'
       FNAMP = 'O-P'
       SUM   = 'SUMX'
       SUM2  = 'SUMX2'

   if fonction=='bcorr':
       FNAM  = 'O-P'
       FNAMP = 'BCORR'
      #FNAMP = 'STATB'  pierre???
       SUM   = 'sumStat'
       SUM2  = 'sumStat2'
   if fonction=='oma':
       FNAM  = 'O-A'
       FNAMP = 'O-A'
       SUM   = 'SUMY'
       SUM2  = 'SUMY2'  
   if fonction=='stdoma':
       FNAM  = 'O-A'
       FNAMP = 'O-A'
       SUM   = 'SUMY'
       SUM2  = 'SUMY2'

   if fonction=='dens':
       FNAM="DENS"
       FNAMP="DENS"
       SUM='N'
       SUM2='N'
   if fonction=='nobs':
       FNAM="DENS"
       FNAMP="DENS"
       SUM='N'
       SUM2='N'
   if fonction=='obs':
       FNAM="OBS"
       FNAMP="OBS"
       SUM='SUMZ'
       SUM2='SUMZ2'

   return FNAM, FNAMP, SUM, SUM2




import os
import sqlite3
import pandas as pd
import matplotlib.pyplot as plt
from glob import glob

# Function to extract date from filename
def extract_date_from_filename(filename, family):
    basename = os.path.basename(filename)
    date_str = basename.split(family)[0][:10]  # 'yyyymmddhh'
    return pd.to_datetime(date_str, format='%Y%m%d%H')

# Function to get counts over time for both paths
def get_counts_by_date(path, family, start_date_str, end_date_str):
    # Convert to datetime for filtering
    start_date = pd.to_datetime(start_date_str, format='%Y%m%d%H')
    end_date = pd.to_datetime(end_date_str, format='%Y%m%d%H')
    
    # Get all files that match the pattern
    files = glob(os.path.join(path, f'*{family}*'))
    print (files)
    # Filter files based on start and end date
    filtered_files = []
    for file in files:
        
        file_date = extract_date_from_filename(file, family)
        if start_date <= file_date <= end_date:
            filtered_files.append(file)

    date_counts = {}

    # Iterate over the filtered files
    for file in filtered_files:
        conn = sqlite3.connect(file)
        cursor = conn.cursor()
        
        # Perform the count(*) query
        cursor.execute('SELECT COUNT(*) FROM data')
        count = cursor.fetchone()[0]
        
        # Extract the date from the filename
        file_date = extract_date_from_filename(file, family)
        
        # Sum counts by date
        if file_date not in date_counts:
            date_counts[file_date] = 0
        date_counts[file_date] += count
        
        conn.close()

    return date_counts

# Paths for the two families
name1 = 'postalt'
#name1 = 'evalalt'

#path1 = f'/home/sprj700/data_maestro/ppp6/maestro_archives/G2FC900V2E22/monitoring/banco/{name1}/'
path1 = f'/home/smco500/.suites/psmon/monitoring/g2/hub/ppp5/sqlite/banco/{name1}/'
path1 = f'/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/{name1}/'
name2 = 'evalalt'
#name2 = 'dbase'

#path2 = f'/home/sprj700/data_maestro/ppp6/maestro_archives/G2FC900V2E22/monitoring/banco/{name2}/'
path2 = f'/home/smco500/.suites/psmon/monitoring/g2/hub/ppp5/sqlite/banco/{name2}/'
path2 = f'/home/smco500/.suites/gdps/g2/hub/ppp6/monitoring/banco/{name2}/'

family1 = '*cris*'
family2 = '*cris*'
print (path1)
print (path2)
start_date_str = '2024102400'  # Example start date
end_date_str = '2024102800'    # Example end date
if name1=='postalt'and name2=='evalalt':

  color1='tab:blue'
  color2='tab:green'

if name1=='evalalt' and name2=='dbase':

  color1='tab:green'
  color2='tab:orange'


# Get counts for both families
counts_family1 = get_counts_by_date(path1, family1, start_date_str, end_date_str)
counts_family2 = get_counts_by_date(path2, family2, start_date_str, end_date_str)

# Create DataFrame for easy plotting
df_family1 = pd.DataFrame(list(counts_family1.items()), columns=['Date', 'Count_family1'])
df_family2 = pd.DataFrame(list(counts_family2.items()), columns=['Date', 'Count_family2'])

# Merge both DataFrames on Date
df_merged = pd.merge(df_family1, df_family2, on='Date', how='outer').fillna(0)
# Sort the DataFrame by Date to ensure correct order in the plot
df_merged = df_merged.sort_values(by='Date')

# Calculate percentage of family2 compared to family1
print (df_merged['Count_family1'],df_merged['Count_family2'])
df_merged['Percentage'] = (df_merged['Count_family1'] / df_merged['Count_family2']) * 100
# Plotting
fig, ax1 = plt.subplots(figsize=(10, 6))

# Plot percentage on primary y-axis
ax1.plot(df_merged['Date'], df_merged['Percentage'], marker='o', color='tab:red', label=f'Percentage ({name2} / {name1})')
print  (df_merged['Percentage'])
ax1.set_xlabel('Date')
ax1.set_ylabel('Percentage (%)', color='tab:red')
ax1.tick_params(axis='y', labelcolor='tab:red')
ax1.set_ylim(0,100)

# Secondary y-axis to show total counts for both family1 and family2
ax2 = ax1.twinx()
ax2.set_ylim(0,400000000)

ax2.plot(df_merged['Date'], df_merged['Count_family1'], marker='x', color=color1, label=f'Count {name1}')
ax2.plot(df_merged['Date'], df_merged['Count_family2'], marker='x', color=color2, label=f'Count {name2} ')
ax2.set_ylabel('Observations (Count)', color='black')

# Adding legends
ax1.legend(loc='lower left', bbox_to_anchor=(0, 1.15))
ax2.legend(loc='lower right', bbox_to_anchor=(0, 1.15))

# Set grid, rotate x labels and tight layout
plt.grid(True)
plt.xticks(rotation=45)
plt.title(f'Percentage of _{name1} over {name2} Count with Total Observations Over Time( Family:{family1})')
plt.tight_layout()
plt.savefig(f'{family1}_{name2}_{name1}_{start_date_str}_{end_date_str}_comparison_plot2.png', dpi=300)  # You can adjust the dpi for higher quality

plt.show()



"""
pikobs.timeserie Module
=======================

The ``timeserie`` module within the ``pikobs`` package is designed for advanced time-series analysis of meteorological observations. It enables users to visualize and analyze both the **quantity** and **quality** of observations across customizable time periods, making it suitable for robust scientific and operational studies.

Main Features
-------------

- **Comprehensive Visualization:** Generate time-series plots for observation counts, bias, and standard deviation.
- **Flexible Grouping Options:** Analyze data per station, aggregated across all stations, or compare different configurations and satellites.

Example Visualizations
----------------------

- **Aggregated statistics for all stations in a single experience (id_stn=join):**

  .. image:: ../../../docs/source/_static/timeserie1.png

  Visualizes time-series data for the ensemble station across the entire experience.

- **Disaggregated station statistics (id_stn=all):**

  .. image:: ../../../docs/source/_static/timeserie2.png

  Displays individual station statistics for more granular comparison.

- **Fully aggregated view across an experience (id_stn=all , all-station aggregation):**

  .. image:: ../../../docs/source/_static/timeserie3.png

  Shows global statistics summarizing all stations within the experience.

Generated Graphs
----------------

1. **Bias and Standard Deviation**
   - **Bias:** Indicates persistent differences between observations and reference data, highlighting systematic errors.
   - **Standard Deviation:** Represents the variability of observations over time.

2. **Observation Count Over Time**
   - Illustrates the density and availability of observations, helping detect temporal gaps or inconsistencies.

Usage & Integration
-------------------

This module can be executed from the command line, allowing seamless integration into batch workflows. Below is an example Bash script:

.. code-block:: bash

   #!/bin/bash

   # Locate Python executable
   PYTHON_EXEC=$(which python)

   # Run the timeserie analysis
   $PYTHON_EXEC -c 'import pikobs; pikobs.timeserie.arg_call()' \

     --path_experience_files "/path/to/experience_files/" \
   
     --experience_name "ops+NOAA21+GOES19" \
     
     --path_control_files "/path/to/control_files/" \
     
     --control_name "ops" \
     
     --pathwork "onthefly2" \
     
     --datestart "2025041200" \
     
     --dateend "2025041300" \
     
     --region "Monde" \
     
     --family "sw" \
     
     --flags_criteria "assimilee" \
     
     --fonction "omp" \
     
     --id_stn "join" \
     
     --channel "join" \
     
     --n_cpu 40

Parameter Descriptions
----------------------

- **path_experience_files:** Directory containing experience data files.
- **experience_name:** Identifier for the specific experiment.
- **path_control_files:** Directory containing control/reference data.
- **control_name:** Identifier for the control/reference configuration.
- **pathwork:** Working directory where outputs and intermediates are stored.
- **datestart:** Start timestamp for the analysis period (format: YYYYMMDDHH).
- **dateend:** End timestamp for the analysis period (format: YYYYMMDDHH).
- **region:** Geographic area of interest. Valid values include:
  - Monde, PoleNord, PoleSud, AmeriqueduNord, OuestAmeriqueduNord, AmeriqueDuNordPlus, ExtratropiquesNord, HemisphereNord, HemisphereSud, Asie, Europe, Mexique, Canada, BaieDhudson, Arctiquecanadien, EtatsUnis, Tropiques30, Tropiques, Australie, Pacifique, Atlantique.
- **family:** Observation family (e.g., mwhs2, to_amsua, iasi_qc, atms_qc, csr_qc, etc.).
- **flags_criteria:** Filtering criterion (e.g., "assimilee").
- **fonction:** Computational function used (e.g., "omp").
- **id_stn:** Specifies stations or satellites to include (e.g., "all", "join", or specific satellite IDs like METOP-1).
- **channel:** Specifies channels to analyze ("all", "join", or specific channel numbers).
- **n_cpu:** Number of CPU cores to use for parallel processing.

Tip: Adjust the command-line arguments to match your data, region, and analysis goals.
"""

import sqlite3
import pikobs
import re
import os
from  dask.distributed import Client
import numpy as np
import sqlite3
import os
import re
import sqlite3
from datetime import datetime, timedelta

def create_table_if_not_exists(cursor):
    """
    Create the 'serie_cardio' table if it does not already exist.

    Args:
        cursor (sqlite3.Cursor): Database cursor for executing SQL queries.
    """
    query = """
        CREATE TABLE IF NOT EXISTS timeserie (
              DATE INTEGER,
                varno INTEGER,
                Nrej INTEGER,
                Nacc INTEGER,
                SUMx FLOAT,
                SUMx2 FLOAT,
                SUMy FLOAT,
                SUMy2 FLOAT,
                SUMz FLOAT,
                SUMz2 FLOAT,
                sumStat FLOAT,
                sumStat2 FLOAT,
                N INTEGER,
                id_stn TEXT,
                vcoord FLOAT,
                flag INTEGER
        ); """
    cursor.execute(query)



def combine(pathfileout: str, filememory: str) -> None:
    """
    Combine multiple SQLite files into a single output file.

    This function copies records from the 'timeserie' table of a source SQLite file 
    (filememory) and inserts them into the 'timeserie' table of a destination/output SQLite file (pathfileout).
    
    Args:

        pathfileout (str): Path to the output SQLite file where combined data will be stored.
        filememory  (str): Path to the source SQLite file (can be memory or disk).
        
    Returns:
        None. The result is saved directly in the specified output file.
    """
    insert_sql = """
        INSERT INTO timeserie(
             DATE, varno, Nrej, Nacc, SUMx, SUMx2, SUMy, SUMy2, SUMz,SUMz2, sumStat,sumStat2,  N,id_stn, vcoord,flag
        )
        SELECT 
             DATE, varno, Nrej, Nacc, SUMx, SUMx2, SUMy, SUMy2, SUMz,SUMz2, sumStat,sumStat2,  N,id_stn, vcoord,flag
        FROM this_avg_db.timeserie
    """
    # Use 'with' statement to ensure connection is properly closed
    with sqlite3.connect(pathfileout, uri=True, isolation_level=None, timeout=9999) as conn:
        try:
            # Set performance-optimized PRAGMAs, provided your application allows for non-durable writes
            conn.execute("PRAGMA journal_mode=OFF;")
            conn.execute("PRAGMA synchronous=OFF;")

            # Ensure the table exists
            create_table_if_not_exists(conn)

            # Attach the source SQLite database
            conn.execute(f"ATTACH DATABASE '{filememory}' AS this_avg_db;")

            # Copy data
            conn.execute(insert_sql)
            conn.commit()

            # Detach the attached database
            conn.execute("DETACH DATABASE this_avg_db;")
        except sqlite3.Error as e:
            print(f"Error combining SQLite files: {e}")
            raise

def create_timeserie_table(
    family, 
    new_db_filename,
    existing_db_filename, 
    region_seleccionada, 
    selected_flags,
    varnos,
    channel,
    id_stn
):
    """
    Function to create a timeserie table in a SQLite database.
    
    Parameters:
    - family: Family identifier.
    - new_db_filename: File path for the new database.
    - existing_db_filename: File path for the existing database.
    - region_seleccionada: Selected region identifier.
    - selected_flags: Flag criteria for selection.
    - varnos: Variable numbers to consider.
    - channel: Channel information.
    - id_stn: Station identifier.
    """

    # Extract date from existing_db_filename
    pattern = r'(\d{10})'
    match = re.search(pattern, existing_db_filename)
    if not match:
        print("No 10 digits found in the filename.")
        return
    date = match.group(1)

    # Create a new temporary database for inspection
    in_memory_db = f"file::memory:?cache=shared"


    with sqlite3.connect(in_memory_db, uri=True, isolation_level=None, timeout=9999999) as new_db_conn:
        new_db_cursor = new_db_conn.cursor()

        # Attach existing database
        new_db_cursor.execute(f"ATTACH DATABASE '{existing_db_filename}' AS db;")
        # Check for required tables and columns
        tables_needed = ['header', 'DATA']
        for table in tables_needed:
            new_db_cursor.execute(f"PRAGMA table_info({table});")
            table_info = new_db_cursor.fetchall()
            if not table_info:
                raise sqlite3.Error(f"Table '{table}' is missing in the database.")

        new_db_conn.enable_load_extension(True)
        extension_dir = f"{os.path.dirname(pikobs.__file__)}/extension/libudfsqlite-shared.so"
        new_db_cursor.execute(f"SELECT load_extension('{extension_dir}')")

        # Configure performance settings
        pragmas = [
            "PRAGMA journal_mode = OFF",
            "PRAGMA synchronous = OFF",
            "PRAGMA foreign_keys = OFF"
        ]
        for pragma in pragmas:
            new_db_cursor.execute(pragma)

        # Collect criteria
        FAM, VCOORD, VCOCRIT, STATB, element, VCOTYP = pikobs.family(family)
        LAT1, LAT2, LON1, LON2 = pikobs.regions(region_seleccionada)
        LATLONCRIT = pikobs.generate_latlon_criteria(LAT1, LAT2, LON1, LON2)
        flag_criteria = pikobs.flag_criteria(selected_flags)

        if varnos:
            element = ",".join(varnos)

        group_conditions = {
            ('join', 'all'): ('"join" as Chan,', 'id_stn as id_stn,', 'group by date,id_stn,varno'),
            ('all', 'join'): (f'{VCOORD} as Chan,', '"join" as id_stn,', f'group by date,{VCOORD},varno'),
            ('all', 'all'): (f'{VCOORD} as Chan,', 'id_stn as id_stn,', f'group by date,id_stn,{VCOORD},varno'),
            ('join', 'join'): ('"join" as Chan,', '"join" as id_stn,', 'group by date,varno')}
        group_channel, group_id_stn, group_id_stn_vcoord = group_conditions.get(
            (channel, id_stn),
            ('', '', '')
        )

        # Check if 'bias_corr' exists
        new_db_cursor.execute("PRAGMA table_info('DATA')")
        columns = new_db_cursor.fetchall()
        bias_corr_exists = any(col[1] == 'bias_corr' for col in columns)

        bias_corr_sum = "sum(bias_corr)" if bias_corr_exists else "NULL"
        bias_corr_sq_sum = "sum(bias_corr * bias_corr)" if bias_corr_exists else "NULL"

        # Create the timeserie table
        new_db_cursor.execute("""
            CREATE TABLE IF NOT EXISTS timeserie (
                DATE INTEGER,
                varno INTEGER,
                Nrej INTEGER,
                Nacc INTEGER,
                SUMx FLOAT,
                SUMx2 FLOAT,
                SUMy FLOAT,
                SUMy2 FLOAT,
                SUMz FLOAT,
                SUMz2 FLOAT,
                sumStat FLOAT,
                sumStat2 FLOAT,
                N INTEGER,
                id_stn TEXT,
                vcoord FLOAT,
                flag INTEGER
            );
        """)
        new_db_conn.commit()

        # Insert data into timeserie table
        query = f"""
        INSERT INTO timeserie (
            DATE, varno, Nrej, Nacc, SUMx, SUMx2, SUMy, SUMy2, SUMz, SUMz2,
            sumStat, sumStat2, N, id_stn, VCOORD, flag
        )
        SELECT
            isodatetime({date}) AS DATE,
            varno AS VARNO,
            count(*) - SUM(flag & 4096 = 4096) AS Nrej,
            SUM(flag & 4096 = 4096) AS Nacc,
            SUM(OMP) AS SUMx,
            SUM(OMP * OMP) AS SUMx2,
            SUM(OMA) AS SUMy,
            SUM(OMA * OMA) AS SUMy2,
            SUM(obsvalue) AS SUMz,
            {bias_corr_sum} AS sumStat,
            SUM(obsvalue * obsvalue) AS SUMz2,
            {bias_corr_sq_sum} AS sumStat2,
            count(*) AS N,
            {group_id_stn}
            {group_channel}
            flag AS flag
        FROM
            db.header
        NATURAL JOIN
            db.DATA
        WHERE 
            varno IN ({element}) AND
            obsvalue IS NOT NULL
            {flag_criteria}
            {LATLONCRIT}
        {group_id_stn_vcoord}
        """
        new_db_cursor.execute(query)
        new_db_conn.commit()

        # Finalize and inspect
        try:
            combine(new_db_filename, in_memory_db)
        except sqlite3.Error as error:
            print(f"Error when combining SQLite files: {error}")


def create_data_list(datestart1, dateend1, families, input_paths,names, pathwork, flag_criteria, regions, varnos):
    data_list = []

    
    # Convert datestart and dateend to datetime objects
    datestart = datetime.strptime(datestart1, '%Y%m%d%H')
    dateend = datetime.strptime(dateend1, '%Y%m%d%H')

    # Initialize the current_date to datestart
    current_date = datestart

    # Define a timedelta of 6 hours
    delta = timedelta(hours=6)
   # FAM, VCOORD, VCOCRIT, STATB, element, VCOTYP = pikobs.family(family)
    
    #flag_criteria = generate_flag_criteria(flag_criteria)

    #element_array = np.array([float(x) for x in element.split(',')])  
    # Iterate through the date range in 6-hour intervals
    while current_date <= dateend: 
      # for varno in element_array:
     for family in families:
      for region in regions:
       for name, input_path in zip(names, input_paths):
        FAM, VCOORD, VCOCRIT, STATB, element, VCOTYP = pikobs.family(family)
        element_array = np.array([float(x) for x in element.split(',')])  
   

        # Format the current date as a string
        formatted_date = current_date.strftime('%Y%m%d%H')

        # Build the file name using the date and family
        filename = f'{formatted_date}_{family}'
        # Create a new dictionary and append it to the list
        data_dict = {
            'family': family,
            'filein': f'{input_path}/{filename}',
            'db_new': f'{pathwork}/{family}/timeserie_{name}_{datestart1}_{dateend1}_{flag_criteria}_{family}_{region}.db',
            'region': region,
            'flag_criteria': flag_criteria,
            'varnos': varnos,
          #  'varno'   : varno
        }
        data_list.append(data_dict)

        # Update the current_date in the loop by adding 6 hours
     current_date += delta

    return data_list

def create_data_list_plot(datestart1,
                          dateend1, 
                          families,
                          namein, 
                          pathwork, 
                          flag_criteria, 
                          regions, 
                          id_stn, 
                          channel,
                          fonctions):
 data_list_plot = []
 for fonction in fonctions:
  for family in families:
   for region in regions:
    
    filea = f'{pathwork}/{family}/timeserie_{namein[0]}_{datestart1}_{dateend1}_{flag_criteria}_{family}_{region}.db'
    namea = namein[0]
    fileset = [filea]
    nameset = [namein[0]]
   
    if len(namein)>1:
          fileb = f'{pathwork}/{family}/timeserie_{namein[1]}_{datestart1}_{dateend1}_{flag_criteria}_{family}_{region}.db'
          fileset = [filea,fileb]
          nameset = [namein[0], namein[1]] 
          conn = sqlite3.connect(fileb)
          cursor = conn.cursor()

    else:
    
      conn = sqlite3.connect(filea)
      cursor = conn.cursor()
    if id_stn=='all':
           query = "SELECT DISTINCT id_stn FROM timeserie;"
           cursor.execute(query)
           id_stns = np.array([item[0] for item in cursor.fetchall()])
           query = "SELECT DISTINCT varno FROM timeserie;"
           cursor.execute(query)
           varnos = np.array([item[0] for item in cursor.fetchall()])
           for varno in varnos:
              data_dict_plot = {
               'id_stn': 'all_t',
               'vcoord': 'join',
               'files_in':fileset,
               'varno':varno,
               'name_in':nameset,
               'region':region,
               'family':family,
               'fonction':fonction}
              data_list_plot.append(data_dict_plot)
    else:
           id_stns = ['join']
    
    for idstn in id_stns:
        if id_stn=='join':
            criter = '   '
        else:
            criter =f'where id_stn = "{idstn}"'
        if channel =='all': 
            query = f"SELECT DISTINCT vcoord, varno FROM timeserie {criter} ORDER BY vcoord ASC;"
            cursor.execute(query)
            vcoords = cursor.fetchall()
            for vcoord, varno in vcoords:
              data_dict_plot = {
               'id_stn': idstn,
               'vcoord': vcoord,
               'files_in':fileset,
               'varno':varno,
               'name_in':nameset,
               'region':region,
               'family':family,
               'fonction':fonction}
              data_list_plot.append(data_dict_plot)
        else:
            query = f"SELECT DISTINCT  varno FROM timeserie ;"
            cursor.execute(query)
            channels_varno = []
            result = cursor.fetchall()
            if result:
                   channels_varno.append(result)
   
            for  varno in channels_varno[0]:
              data_dict_plot = {
               'id_stn': idstn,
               'vcoord': 'join',
               'files_in':fileset,
               'varno':varno,
               'name_in':nameset,
               'region':region,
               'family':family,
               'fonction':fonction}
              data_list_plot.append(data_dict_plot)
    

  return data_list_plot


def make_timeserie(files_in,
                   names_in,  
                   pathwork, 
                   datestart,
                   dateend,
                   regions, 
                   familys, 
                   flag_criteria, 
                   fonction,
                   varnos,
                   id_stn,
                   channel,
                   n_cpu):


    for family in familys:
       pikobs.delete_create_folder(pathwork, family)

          
    data_list = create_data_list(datestart,
                                 dateend, 
                                 familys, 
                                 files_in,
                                 names_in,
                                 pathwork,
                                 flag_criteria, 
                                 regions,
                                 varnos)
          

    import time
    import dask
    t0 = time.time()
    if n_cpu==1:
           for  data_ in data_list:  
               print (f"Serie = {len(data_list)}")
               #print (data_['family'], 
               create_timeserie_table(data_['family'], 
                                      data_['db_new'], 
                                      data_['filein'],
                                      data_['region'],
                                      data_['flag_criteria'],
                                      data_['varnos'],
                                      channel,
                                      id_stn)
               
       
       
       
       
    else:
           print (f'in Parallel  = {len(data_list)}')
           with dask.distributed.Client(processes=True, threads_per_worker=1, 
                                              n_workers=n_cpu, 
                                              silence_logs=40) as client:
               delayed_funcs = [dask.delayed(create_timeserie_table)(data_['family'], 
                                                 data_['db_new'], 
                                                 data_['filein'],
                                                 data_['region'],
                                                 data_['flag_criteria'],
                                                 data_['varnos'],
                                                 channel,
                                                 id_stn)for data_ in data_list]
               results = dask.compute(*delayed_funcs)
    
    tn= time.time()
    print ('Total time:', round(tn-t0,2) )  
    data_list_plot = create_data_list_plot(datestart,
                                       dateend, 
                                       familys, 
                                       names_in, 
                                       pathwork,
                                       flag_criteria, 
                                       regions,
                                       id_stn,
                                       channel,
                                       fonction)

    #$ os.makedirs(f'{pathwork}/timeserie')
    fig_title = ''
    t0 = time.time()
    #n_cpu=1
    if n_cpu==1:
           for  data_ in data_list_plot[0:1]:  
               print ("Plotting in serie")

               pikobs.timeserie_plot(
                                  pathwork,
                     datestart,
                     dateend,
                     data_['fonction'],
                     flag_criteria,
                     data_['family'],
                     data_['region'],
                     fig_title,
                     data_['vcoord'], 
                     data_['id_stn'], 
                     data_['varno'],
                     data_['files_in'],
                     data_['name_in'])  
    else:
             print (f'Plotting in Parallel = {len(data_list_plot)}')
             with dask.distributed.Client(processes=True, threads_per_worker=1, 
                                              n_workers=n_cpu, 
                                              silence_logs=40) as client:
               delayed_funcs = [dask.delayed(pikobs.timeserie_plot)(
                                  pathwork,
                     datestart,
                     dateend,
                     data_['fonction'],
                     flag_criteria,
                     data_['family'],
                     data_['region'],
                     fig_title,
                     data_['vcoord'], 
                     data_['id_stn'], 
                     data_['varno'],
                     data_['files_in'],
                     data_['name_in'])for data_ in data_list_plot]

               results = dask.compute(*delayed_funcs)
    tn= time.time()
    print ('Total time:', round(tn-t0,2) )  
 



def arg_call():
    import argparse
    import sys
    parser = argparse.ArgumentParser()
    parser.add_argument('--path_control_files', default='undefined', type=str, help="Directory where input sqlite files are located")
    parser.add_argument('--control_name', default='undefined', type=str, help="Directory where input sqlite files are located")
    parser.add_argument('--path_experience_files', default='undefined', type=str, help="Directory where input sqlite files are located")
    parser.add_argument('--experience_name', default='undefined', type=str, help="Directory where input sqlite files are located")
    parser.add_argument('--pathwork', default='undefined', type=str, help="Working directory")
    parser.add_argument('--datestart', default='undefined', type=str, help="Start date")
    parser.add_argument('--dateend', default='undefined', type=str, help="End date")
    parser.add_argument('--region', nargs="+", default='undefined', type=str, help="Region")
    parser.add_argument('--family', nargs="+", default='undefined', type=str, help="Family")
    parser.add_argument('--flags_criteria', default='undefined', type=str, help="Flags criteria")
    parser.add_argument('--fonction', nargs="+", default='undefined', type=str, help="Function")
    parser.add_argument('--varnos', nargs="+", default='undefined', type=str, help="Function")
    parser.add_argument('--id_stn', default='all', type=str, help="id_stn") 
    parser.add_argument('--channel', default='all', type=str, help="channel")
    parser.add_argument('--n_cpus', default=1, type=int, help="Number of CPUs")

    args = parser.parse_args()
    for arg in vars(args):
       print (f'--{arg} {getattr(args, arg)}')
    # Check if each argument is 'undefined'
    if args.path_control_files == 'undefined':
        files_in = [args.path_experience_files]
        names_in = [args.experience_name]
    else:    
        if args.path_experience_files == 'undefined':
            raise ValueError('You must specify --path_experience_files')
        if args.experience_name == 'undefined':
            raise ValueError('You must specify --experience_name')
        else:

            files_in = [args.path_control_files, args.path_experience_files]
            names_in = [args.control_name, args.experience_name]
    if args.varnos == 'undefined':
        args.varnos = []
    if args.pathwork == 'undefined':
        raise ValueError('You must specify --pathwork')
    if args.datestart == 'undefined':
        raise ValueError('You must specify --datestart')
    if args.dateend == 'undefined':
        raise ValueError('You must specify --dateend')
    if args.region == 'undefined':
        raise ValueError('You must specify --region')
    if args.family == 'undefined':
        raise ValueError('You must specify --family')
    if args.flags_criteria == 'undefined':
        raise ValueError('You must specify --flags_criteria')
    if args.fonction == 'undefined':
        raise ValueError('You must specify --fonction')


    # Comment
    # Proj='cyl' // Proj=='OrthoN'// Proj=='OrthoS'// Proj=='robinson' // Proj=='Europe' // Proj=='Canada' // Proj=='AmeriqueNord' // Proj=='Npolar' //  Proj=='Spolar' // Proj == 'reg'
  
    # Call your function with the arguments
    sys.exit(make_timeserie(files_in,
                            names_in, 
                            args.pathwork,
                            args.datestart,
                            args.dateend,
                            args.region,
                            args.family,
                            args.flags_criteria,
                            args.fonction, 
                            args.varnos,
                            args.id_stn,
                            args.channel,
                            args.n_cpus))

if __name__ == '__main__':
    args = arg_call()    
